import json as json_handler
import traceback
from importlib import import_module
from balebot.models.constants.errors import Error
from balebot.utils.logger import Logger

from balebot.models.server_responses.bot_error_response import BotError
from balebot.models.server_responses.bot_success_response import BotSuccess


class Response:
    def __init__(self, json):
        self.logger = Logger.get_logger()

        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        self._id = json_dict.get("id", None)
        self.body_json = json_dict.get("body", None)

        if self.body_json.get("tag", None):
            self.body = BotError(self.body_json)
            del self.body_json
        else:
            self.body = None

    def is_bot_error(self):
        return isinstance(self.body, BotError)

    def create_body(self, body_module_name, body_class_name):
        try:
            if self.body is None:
                if self.body_json.get("tag", None):
                    self.body = BotError(self.body_json)
                elif not self.body_json:
                    self.body = BotSuccess()
                else:
                    module_obj = import_module(body_module_name)
                    body_class = getattr(module_obj, body_class_name)
                    self.body = body_class(self.body_json)
                del self.body_json

        except Exception as ex:
            self.logger.error(ex, extra={"tag": "err"})
            traceback.print_exc()

    @property
    def id(self):
        return self._id

    def get_response(self):
        return self.body_json



import json as json_handler

from balebot.models.messages.base_message import BaseMessage
from balebot.models.constants.errors import Error

from balebot.models.constants.message_type import MessageType


class TemplateResponseMessage(BaseMessage):
    def __init__(self, text, template_message_response_id):
        self.text = str(text)
        self.text_message = self.text
        self.template_message_response_id = str(template_message_response_id)

    def get_json_object(self):
        data = {
            "$type": MessageType.template_response_message,
            "textMessage": self.text,
            "templateMessageResponseId": self.template_message_response_id,
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        text_message = json_dict.get("textMessage", None)
        template_message_response_id = json_dict.get("templateMessageResponseId", None)

        if (text_message is None) or (template_message_response_id is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(text=text_message, template_message_response_id=template_message_response_id)



import json as json_handler

from balebot.models.base_models.raw_json import RawJson
from balebot.models.messages.base_message import BaseMessage
from balebot.models.constants.errors import Error
from balebot.models.constants.raw_json_type import RawJsonType
from balebot.models.constants.message_type import MessageType


class ContactMessage(RawJson, BaseMessage):
    def __init__(self, name, emails, phones):
        self.name = str(name)

        if isinstance(emails, list):
            self.emails = [str(email) for email in emails]
        else:
            raise ValueError(Error.unacceptable_object_type)

        if isinstance(phones, list):
            self.phones = ["".join(str(phone).split(" ")) for phone in phones]
        else:
            raise ValueError(Error.unacceptable_object_type)

    def get_json_object(self):

        data = {
            "$type": MessageType.json_message,
            "rawJson": json_handler.dumps({
                "dataType": RawJsonType.contact,
                "data": {
                    RawJsonType.contact: {
                        "name": self.name,
                        "emails": self.emails,
                        "phones": self.phones
                    }
                }
            })
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        json_dict = json_handler.loads(json_dict.get('rawJson'))
        data = json_dict.get('data', None)
        contact = data.get(RawJsonType.contact, None)
        name = contact.get('name', None)
        emails = contact.get('emails', None)
        phones = contact.get('phones', None)

        return cls(name=name, emails=emails, phones=phones)



from balebot.models.constants.message_type import MessageType
from balebot.models.messages.banking import bank_message
from balebot.models.messages.banking import purchase_message
from balebot.models.messages import document_message
from balebot.models.messages import location_message
from balebot.models.messages import contact_message
from balebot.models.messages import photo_message
from balebot.models.messages import sticker_message
from balebot.models.messages.template import template_message
from balebot.models.messages import text_message
from balebot.models.messages import unsupported_message
from balebot.models.messages import json_message
from balebot.models.messages import video_message
from balebot.models.messages import voice_message
from balebot.models.messages import template_response_message


class MessageFactory:
    @staticmethod
    def create_message(json_dict):
        message_type = json_dict.get("$type")

        if message_type == MessageType.text_message:
            return text_message.TextMessage.load_from_json(json_dict)

        elif message_type == MessageType.template_message:
            return template_message.TemplateMessage.load_from_json(json_dict)

        elif message_type == MessageType.purchase_message:
            return purchase_message.PurchaseMessage.load_from_json(json_dict)

        elif message_type == MessageType.unsupported_message:
            return unsupported_message.UnsupportedMessage.load_from_json(json_dict)

        elif message_type == MessageType.sticker_message:
            return sticker_message.StickerMessage.load_from_json(json_dict)

        elif message_type == MessageType.json_message:

            if json_message.JsonMessage.is_raw_location_message(json_dict):
                return location_message.LocationMessage.load_from_json(json_dict)

            elif json_message.JsonMessage.is_raw_contact_message(json_dict):
                return contact_message.ContactMessage.load_from_json(json_dict)

            return json_message.JsonMessage.load_from_json(json_dict)

        elif message_type == MessageType.template_response_message:
            return template_response_message.TemplateResponseMessage.load_from_json(json_dict)

        elif message_type == MessageType.bank_message:
            return bank_message.BankMessage.load_from_json(json_dict)

        elif message_type == MessageType.document_message:

            if document_message.DocumentMessage.is_raw_document_message(json_dict):
                return document_message.DocumentMessage.load_from_json(json_dict)

            elif document_message.DocumentMessage.is_photo_message(json_dict):
                return photo_message.PhotoMessage.load_from_json(json_dict)

            elif document_message.DocumentMessage.is_video_message(json_dict):
                return video_message.VideoMessage.load_from_json(json_dict)

            elif document_message.DocumentMessage.is_voice_message(json_dict):
                return voice_message.VoiceMessage.load_from_json(json_dict)

        else:
            return None



import json as json_handler

from balebot.models.base_models.jsonable import Jsonable
from balebot.models.constants.errors import Error


class Member(Jsonable):
    def __init__(self, user_id, invite_user_id, date, is_admin=None):
        self.user_id = str(user_id)
        self.invite_user_id = str(invite_user_id)
        self.date = str(date)
        self.is_admin = bool(is_admin) if is_admin else None

    def get_json_object(self):
        data = {
            "userId": self.user_id,
            "inviteUserId": self.invite_user_id,
            "date": self.date,
            "isAdmin": self.is_admin,

        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        user_id = json_dict.get('userId', None)
        invite_user_id = json_dict.get('inviteUserId', None)
        date = json_dict.get('date', None)
        is_admin = json_dict.get('isAdmin', None)

        if (not user_id) or (not invite_user_id) or (date is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(user_id=user_id, invite_user_id=invite_user_id, date=date, is_admin=is_admin)



from balebot.models.messages.template.template_message import TemplateMessage
from balebot.filters.filter import Filter


class TemplateMessageFilter(Filter):
    def match(self, message):
        return isinstance(message, TemplateMessage)



from balebot.filters.filter import Filter
from balebot.models.messages.location_message import LocationMessage


class LocationFilter(Filter):
    def match(self, message):
        if isinstance(message, LocationMessage):
            return self.validator(message) if self.validator else True
        else:
            return False



from balebot.models.base_models.avatar import Avatar
from balebot.models.base_models.bot_api_quoted_message import BotApiQuotedMessage
from balebot.models.base_models.bot_command import BotCommand
from balebot.models.base_models.bot_quoted_message import BotQuotedMessage
from balebot.models.base_models.contact_record import ContactRecord
from balebot.models.base_models.fat_seq_update import FatSeqUpdate
from balebot.models.base_models.file_location import FileLocation
from balebot.models.base_models.group import Group
from balebot.models.base_models.group_member import GroupMember
from balebot.models.base_models.group_peer import GroupPeer
from balebot.models.base_models.image_location import ImageLocation
from balebot.models.base_models.jsonable import Jsonable
from balebot.models.base_models.member import Member
from balebot.models.base_models.pack_info import PackInfo
from balebot.models.base_models.peer import Peer
from balebot.models.base_models.request import Request
from balebot.models.base_models.request_body import RequestBody
from balebot.models.base_models.response import Response
from balebot.models.base_models.response_body import ResponseBody
from balebot.models.base_models.user import User
from balebot.models.base_models.user_peer import UserPeer
from balebot.models.base_models.raw_json import RawJson



class Error:
    none_or_invalid_attribute = "main attributes should have value."
    unacceptable_json = "json input has unacceptable format."
    unacceptable_object_type = "object has unacceptable type"



import logging
import os


class Config:
    receive_timeout = int(os.environ.get("SESSION_TIMEOUT",60))
    base_url = os.environ.get('BASE_URL', "wss://api.bale.ai/v1/bots/")
    request_timeout = int(os.environ.get('REQUEST_TIMEOUT', 5))
    # 0:print to output        1:use graylog       2:both 0 and 1
    use_graylog = os.environ.get('SDK_USE_GRAYLOG', "0")
    source = os.environ.get('LOG_SOURCE', "bot_source")
    graylog_host = os.environ.get('SDK_GRAYLOG_HOST', "172.30.41.67")
    graylog_port = int(os.environ.get('SDK_GRAYLOG_PORT', 12201))
    log_level = int(os.environ.get('SDK_LOG_LEVEL', logging.DEBUG))
    log_facility_name = os.environ.get('SDK_LOG_FACILITY_NAME', "python_bale_bot")
    monitoring_hash = os.environ.get('MONITORING_HASH', "SADJSDSDas4d2asf41f2a2faasd45sas-")
    real_time_fetch_updates = os.environ.get('REAL_TIME_FETCH_UPDATES', True)
    continue_last_processed_seq = os.environ.get('CONTINUE_LAST_PROCESSED_SEQ', False)
    timeInterval = int(os.environ.get('TIME_INTERVAL', 1))  # unit for time interval is second)
    updates_number = int(os.environ.get('UPDATES_NUMBER', 3))
    heartbeat = int(os.environ.get("HEARTBEAT", 30))
    state_holder = int(os.getenv('STATE_HOLDER', 0))
    redis_host = os.getenv('REDIS_HOST', "localhost")
    redis_port = int(os.getenv('REDIS_PORT', 6379))
    redis_db = os.getenv('REDIS_DB', 0)
    group_shield = int(os.getenv('GROUP_SHIELD', 1))



import json as json_handler
from balebot.models.constants.errors import Error

from balebot.models.base_models.peer import Peer
from balebot.models.base_models.jsonable import Jsonable


class BotQuotedMessage(Jsonable):
    def __init__(self, message_id=None, peer=None):

        self.message_id = str(message_id) if message_id else None

        if peer:
            if isinstance(peer, Peer):
                self.peer = peer
            else:
                raise ValueError(Error.unacceptable_object_type)
        else:
            self.peer = None

    def get_json_object(self):
        data = {
            "messageId": self.message_id,
            "peer": self.peer.get_json_object() if self.peer else None,
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        pass



from balebot.models.messages.banking.purchase_message import PurchaseMessage
from balebot.models.messages.banking.bank_message import BankMessage

from balebot.models.messages.template.template_message import TemplateMessage
from balebot.models.messages.template.template_message_button import TemplateMessageButton

from balebot.models.messages.template_response_message import TemplateResponseMessage
from balebot.models.messages.base_message import BaseMessage
from balebot.models.messages.json_message import JsonMessage
from balebot.models.messages.location_message import LocationMessage
from balebot.models.messages.contact_message import ContactMessage
from balebot.models.messages.text_message import TextMessage
from balebot.models.messages.unsupported_message import UnsupportedMessage
from balebot.models.messages.photo_message import PhotoMessage
from balebot.models.messages.document_message import DocumentMessage
from balebot.models.messages.sticker_message import StickerMessage
from balebot.models.messages.video_message import VideoMessage
from balebot.models.messages.voice_message import VoiceMessage



import json as json_handler

from balebot.models.base_models.peer import Peer
from balebot.models.base_models.user_peer import UserPeer
from balebot.models.factories import message_factory
from balebot.models.server_updates.update_body import UpdateBody
from balebot.models.constants.errors import Error

from balebot.models.base_models.bot_api_quoted_message import BotApiQuotedMessage


class MessageUpdate(UpdateBody):
    def __init__(self, json_dict):
        if isinstance(json_dict, dict):
            json_dict = json_dict
        elif isinstance(json_dict, str):
            json_dict = json_handler.loads(json_dict)
        else:
            raise ValueError(Error.unacceptable_json)

        self.peer = Peer.load_from_json(json_dict.get("peer", None))
        self.sender_user = UserPeer.load_from_json(json_dict.get("sender", None))
        self.date = json_dict.get("date", None)
        self.random_id = json_dict.get("randomId", None)
        self.message = message_factory.MessageFactory.create_message(json_dict.get("message", None))
        self.quoted_message = BotApiQuotedMessage(json_dict.get("quotedMessage", None)) if json_dict.get(
            "quotedMessage", None) else None



import json as json_handler
from balebot.models.constants.errors import Error

from balebot.models.base_models.image_location import ImageLocation

from balebot.models.base_models.jsonable import Jsonable


class Avatar(Jsonable):
    def __init__(self, small_image=None, large_image=None, full_image=None):

        if small_image:
            if isinstance(small_image, ImageLocation):
                self.small_image = small_image
            else:
                raise ValueError(Error.unacceptable_object_type)
        else:
            self.small_image = None

        if large_image:
            if isinstance(large_image, ImageLocation):
                self.large_image = large_image
            else:
                raise ValueError(Error.unacceptable_object_type)
        else:
            self.large_image = None

        if full_image:
            if isinstance(full_image, ImageLocation):
                self.full_image = full_image
            else:
                raise ValueError(Error.unacceptable_object_type)
        else:
            self.full_image = None

    def get_json_object(self):

        data = {
            "smallImage": self.small_image.get_json_object(),
            "largeImage": self.large_image.get_json_object(),
            "fullImage": self.full_image.get_json_object(),
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        small_image = ImageLocation.load_from_json(json_dict.get("smallImage", None)) \
            if json_dict.get("smallImage", None) else None
        large_image = ImageLocation.load_from_json(json_dict.get("largeImage", None)) \
            if json_dict.get("largeImage", None) else None
        full_image = ImageLocation.load_from_json(json_dict.get("fullImage", None)) \
            if json_dict.get("fullImage", None) else None

        return cls(small_image=small_image, large_image=large_image, full_image=full_image)



class MessageType:
    text_message = 'Text'
    sticker_message = 'Sticker'
    template_message = 'TemplateMessage'
    purchase_message = 'PurchaseMessage'
    bank_message = 'BankMessage'
    order_message = 'OrderMessage'
    unsupported_message = 'Unsupported'
    json_message = 'Json'
    document_message = 'Document'
    template_response_message = 'TemplateMessageResponse'
    location_message = 'location'
    contact_message = 'contact'



class BankExtTypes:
    bank_money_transfer = "BankMoneyTransfer"
    bank_remain = "BankRemain"
    bank_transaction = "BankTransaction"
    deprecated_receipt = "DeprecatedReceipt"
    deprecated_receipt_message = "DepricatedReceiptMessage"
    receipt_message = "ReceiptMessage"



from balebot.models.messages.text_message import TextMessage
import re
from balebot.filters.filter import Filter


class TextFilter(Filter):
    def __init__(self, keywords=None, pattern=None, validator=None, include_commands=True):
        super(TextFilter, self).__init__(validator)
        self.keywords = []
        if isinstance(keywords, list):
            self.keywords += keywords
        elif isinstance(keywords, str):
            self.keywords.append(keywords)

        self.pattern = pattern
        self.validator = validator if callable(validator) else None
        self.include_commands = include_commands

    def match(self, message):
        if isinstance(message, TextMessage):
            text = message.text
            if not self.include_commands:
                if text.startswith("/"):
                    return False

            if not self.pattern and not self.keywords and not self.validator:
                return True

            if self.find_keywords(text):
                return True
            elif self.find_pattern(text):
                return True
            elif self.validate(text):
                return True
        else:
            return False

    def find_keywords(self, text):
        for keyword in self.keywords:
            if keyword:
                if text.find(keyword) != -1:
                    return True
        return False

    def find_pattern(self, text):
        if self.pattern:
            return re.search(self.pattern, text)
        return False



from balebot.models.messages.contact_message import ContactMessage
from balebot.models.messages.json_message import JsonMessage
from balebot.filters.filter import Filter


class ContactFilter(Filter):
    def match(self, message):
        if isinstance(message, JsonMessage):
            raw_json = message.raw_json
            return isinstance(raw_json, ContactMessage)
        else:
            return False



import json as json_handler

from balebot.models.base_models.bot_quoted_message import BotQuotedMessage
from balebot.models.base_models.peer import Peer
from balebot.models.base_models.request_body import RequestBody
from balebot.models.constants.errors import Error
from balebot.models.constants.request_type import RequestType
from balebot.models.messages.base_message import BaseMessage
from balebot.utils.util_functions import generate_random_id


class SendMessage(RequestBody):
    def __init__(self, message, receiver_peer, quoted_message=None, random_id=None):

        if isinstance(message, BaseMessage) and isinstance(receiver_peer, Peer) and (
                    isinstance(quoted_message, BotQuotedMessage) or not quoted_message):

            self.message = message
            self.receiver_peer = receiver_peer
            self.quoted_message = quoted_message

            if random_id:
                self._random_id = str(random_id)
            else:
                self._random_id = str(generate_random_id())

        else:
            raise ValueError(Error.unacceptable_object_type)

    def get_json_object(self):

        data = {
            "$type": RequestType.send_message,
            "peer": self.receiver_peer.get_json_object(),
            "randomId": self._random_id,
            "message": self.message.get_json_object(),
            "quotedMessage": self.quoted_message.get_json_object() if self.quoted_message else None,
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())



import json as json_handler

from balebot.models.base_models.peer import Peer
from balebot.models.base_models.request_body import RequestBody
from balebot.models.constants.errors import Error
from balebot.models.constants.request_type import RequestType
from balebot.models.messages.base_message import BaseMessage
from balebot.utils.util_functions import generate_random_id


class EditMessage(RequestBody):
    def __init__(self, updated_message, peer, random_id):
        if isinstance(updated_message, BaseMessage) and isinstance(peer, Peer):
            self.updated_message = updated_message
            self.peer = peer
            self._random_id = str(random_id)

        else:
            raise ValueError(Error.unacceptable_object_type)

    def get_json_object(self):

        data = {
            "$type": RequestType.edit_message,
            "peer": self.peer.get_json_object(),
            "randomId": self._random_id,
            "message": self.updated_message.get_json_object()
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())



import json as json_handler

from balebot.models.base_models.avatar import Avatar
from balebot.models.base_models.response_body import ResponseBody
from balebot.models.base_models.value_types.map_value import MapValue
from balebot.models.constants.errors import Error

from balebot.models.base_models.member import Member


class GetGroupApiResponse(ResponseBody):
    def __init__(self, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        self.id = json_dict.get("id", None)
        self.access_hash = json_dict.get("accessHash", None)
        self.title = json_dict.get("title", None)

        self.avatar = Avatar.load_from_json(json_dict.get("avatar", None)) if json_dict.get("avatar", None) else None

        self.is_member = json_dict.get("isMember", None)
        self.creator_user_id = json_dict.get("creatorUserId", None)

        self.members = [Member.load_from_json(member) for member in json_dict.get("members", None)] if json_dict.get(
            "members", None) else None

        self.create_date = json_dict.get("createDate", None)
        self.is_admin = json_dict.get("isAdmin", None)
        self.theme = json_dict.get("theme", None)
        self.about = json_dict.get("about", None)
        self.is_hidden = json_dict.get("isHidden", None)
        self.ext = MapValue.load_from_json(json_dict.get("ext", None)) if json_dict.get("ext", None) else None
        self.members_count = json_dict.get("membersCount", None)
        self.group_type = json_dict.get("groupType", None)["$type"]
        self.can_send_message = json_dict.get("canSendMessage", None)
        self.nick = json_dict.get("nick", None)
        self.became_orphaned = json_dict.get("becameOrphaned", None)
        self.state_version = json_dict.get("stateVersion", None)



import json as json_handler

from balebot.models.base_models.request_body import RequestBody
from balebot.models.constants.request_type import RequestType


class GetDifference(RequestBody):
    def __init__(self, seq, how_many):
        self.seq = int(seq) if seq else 0
        self.how_many = how_many

    def get_json_object(self):
        data = {
            "$type": RequestType.get_difference,
            "seq": self.seq,
            "howMany": self.how_many,
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())



from balebot.handlers.handler import Handler
from balebot.models.base_models.fat_seq_update import FatSeqUpdate
from balebot.models.messages.template_response_message import TemplateResponseMessage
from balebot.models.messages.text_message import TextMessage


class CommandHandler(Handler):
    def __init__(self, commands, callback, include_template_response=False):
        super(CommandHandler, self).__init__(callback=callback)
        if isinstance(commands, str):
            commands = [commands.lower()]
        elif isinstance(commands, list):
            commands = [command.lower() for command in commands]
        self.commands = [(command[1:] if command.startswith("/") else command) for command in commands]
        self.include_template_response = include_template_response

    def check_update(self, update):
        if isinstance(update, FatSeqUpdate) and update.is_message_update():
            message = update.get_effective_message()
            if isinstance(message, TextMessage):
                message_text = message.text.lower()
            elif self.include_template_response and isinstance(message, TemplateResponseMessage):
                message_text = message.text_message.lower()
            else:
                return False

            if not message_text.startswith("/"):
                return False

            message_command = message_text[1:].split(" ")[0]

            return any(message_command == command for command in self.commands)
        return False

    def handle_update(self, dispatcher, update):
        return self.callback(bot=dispatcher.bot, update=update)

    def is_default_handler(self):
        return False



import json as json_handler

from balebot.models.messages.document_message import DocumentMessage

from balebot.models.constants.document_type import DocumentType
from balebot.models.messages.text_message import TextMessage


class PhotoMessage(DocumentMessage):
    def __init__(self, file_id, access_hash, name, file_size, mime_type, thumb, width=80, height=80, ext_width=None,
                 ext_height=None, file_storage_version=None, caption_text=None, checksum=None, algorithm=None):

        super(PhotoMessage, self).__init__(file_id, access_hash, name, file_size, mime_type,
                                           caption_text, checksum, algorithm, file_storage_version)
        self.thumb = str(thumb)
        self.width = int(width)
        self.height = int(height)
        self.ext_width = int(ext_width) if ext_width else width
        self.ext_height = int(ext_height) if ext_height else height

    def get_json_object(self):

        data = super().get_json_object()
        data["thumb"] = {
            "width": self.width,
            "height": self.height,
            "thumb": self.thumb
        }
        data["ext"] = {
            "$type": DocumentType.photo_document,
            "width": self.ext_width,
            "height": self.ext_height,
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError("json input has unacceptable format.")

        file_id = json_dict.get('fileId', None)
        access_hash = json_dict.get('accessHash', None)
        name = json_dict.get('name', None)
        file_size = json_dict.get('fileSize', None)
        mime_type = json_dict.get('mimeType', None)

        ext = json_dict.get('ext', None)
        ext_width = ext.get('width', None)
        ext_height = ext.get('height', None)

        thumb = json_dict.get('thumb', None)
        width = thumb.get('width', None)
        height = thumb.get('height', None)
        thumb_txt = thumb.get('thumb', None)

        file_storage_version = json_dict.get('fileStorageVersion', None)

        caption_message = TextMessage.load_from_json(json_dict.get('caption', None))

        checksum = json_dict.get('checkSum', None)
        algorithm = json_dict.get('algorithm', None)

        if (not file_id) or (not access_hash) or (name is None) or (file_size is None) or \
                (mime_type is None) or (width is None) or (height is None) or \
                (ext_width is None) or (ext_height is None) or (thumb_txt is None):
            raise ValueError("main parameters should have value.")

        return cls(file_id=file_id, access_hash=access_hash, name=name, file_size=file_size,
                   mime_type=mime_type, width=width, height=height, ext_width=ext_width, ext_height=ext_height,
                   thumb=thumb_txt, file_storage_version=file_storage_version, caption_text=caption_message,
                   checksum=checksum, algorithm=algorithm)



import json as json_handler


class TemplateMessageButton:
    # text: str
    # value: str
    # action: int

    def __init__(self, text, value=None, action=None):
        self.text = str(text)
        self.value = str(value or text)
        self.action = int(action or 0)

    def get_json_object(self):
        data = {
            "text": self.text,
            "value": self.value,
            "action": self.action,
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        pass



import json as json_handler

from balebot.models.base_models.banking.bank_ext import BankExt
from balebot.models.base_models.value_types.map_value import MapValue
from balebot.models.constants.bank_ext_types import BankExtTypes
from balebot.models.constants.errors import Error
from balebot.models.factories import message_factory
from balebot.models.messages.base_message import BaseMessage


class DeprecatedReceipt(BankExt):
    def __init__(self, receipt_type, transfer_info, original_message=None):
        self._receipt_type = receipt_type

        if isinstance(transfer_info, MapValue):
            self._transfer_info = transfer_info
        else:
            raise ValueError(Error.unacceptable_object_type)

        if original_message:
            if isinstance(original_message, BaseMessage):
                self._original_message = original_message
            else:
                raise ValueError(Error.unacceptable_object_type)
        else:
            self._original_message = None

    def get_json_object(self):
        data = {
            "$type": BankExtTypes.deprecated_receipt,
            "receiptType": {
                "$type": self._receipt_type,
            },
            "transferInfo": self._transfer_info.get_json_object(),
            "originalMessage": self._original_message.get_json_object() if self._original_message else None,

        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        receipt_type = json_dict.get('receiptType', None)["$type"]
        transfer_info = MapValue.load_from_json(json_dict.get('transferInfo', None))
        original_message = message_factory.MessageFactory.create_message(
            json_dict.get('originalMessage', None)) if json_dict.get(
            'originalMessage', None) else None

        if (not receipt_type) or (transfer_info is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(receipt_type=receipt_type, transfer_info=transfer_info, original_message=original_message)



import json as json_handler

from balebot.models.base_models.banking.bank_ext import BankExt
from balebot.models.base_models.value_types.map_value import MapValue
from balebot.models.constants.bank_ext_types import BankExtTypes
from balebot.models.constants.errors import Error
from balebot.models.factories import message_factory
from balebot.models.messages.base_message import BaseMessage


class DeprecatedReceiptMessage(BankExt):
    def __init__(self, partial_receipt_type, partial_transfer_info, partial_original_message=None,
                 partial_original_message_rid=None):

        self._partial_receipt_type = str(partial_receipt_type)

        if isinstance(partial_transfer_info, MapValue):
            self._partial_transfer_info = partial_transfer_info
        else:
            raise ValueError(Error.unacceptable_object_type)

        if partial_original_message:
            if isinstance(partial_original_message, BaseMessage):
                self._partial_original_message = partial_original_message
            else:
                raise ValueError(Error.unacceptable_object_type)
        else:
            self._partial_original_message = None

        self._partial_original_message_rid = str(partial_original_message_rid) if partial_original_message_rid else None

    def get_json_object(self):
        data = {
            "$type": BankExtTypes.deprecated_receipt_message,
            "partialReceiptType": {
                "$type": self._partial_receipt_type,
            },
            "partialTransferInfo": self._partial_transfer_info.get_json_object(),
            "partialOriginalMessage": self._partial_original_message.get_json_object() if self._partial_original_message
            else None,
            "partialOrginalMessageRID": self._partial_original_message_rid if self._partial_original_message else None,

        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        partial_receipt_type = json_dict.get('partial_receipt_type', None)["$type"]
        partial_transfer_info = MapValue.load_from_json(json_dict.get('partial_transfer_info', None))
        partial_original_message = message_factory.MessageFactory.create_message(
            json_dict.get('partial_original_message', None)) if json_dict.get(
            'partial_original_message', None) else None
        partial_original_message_rid = json_dict.get('partial_original_message_rid', None)

        if (not partial_receipt_type) or (partial_transfer_info is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(partial_receipt_type=partial_receipt_type,
                   partial_transfer_info=partial_transfer_info,
                   partial_original_message=partial_original_message,
                   partial_original_message_rid=partial_original_message_rid)



import json as json_handler

from balebot.models.base_models.request_body import RequestBody
from balebot.models.constants.request_type import RequestType


class CreateGroup(RequestBody):
    def __init__(self, title):
        self.title = str(title)

    def get_json_object(self):
        data = {
            "$type": RequestType.create_group,
            "title": self.title,
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())



import json as json_handler

from balebot.models.base_models.image_location import ImageLocation
from balebot.models.messages.base_message import BaseMessage
from balebot.models.constants.errors import Error

from balebot.models.constants.message_type import MessageType


class StickerMessage(BaseMessage):
    def __init__(self, sticker_id=None, sticker_collection_id=None, sticker_collection_access_hash=None,
                 fast_preview=None, image512=None, image256=None):
        self._sticker_id = str(sticker_id) if sticker_id else None
        self._sticker_collection_id = str(sticker_collection_id) if sticker_collection_id else None
        self._sticker_collection_access_hash = str(
            sticker_collection_access_hash) if sticker_collection_access_hash else None
        self._fast_preview = list(fast_preview) if fast_preview else None

        if image512:
            if isinstance(image512, ImageLocation):
                self._image512 = image512
            else:
                raise ValueError(Error.unacceptable_object_type)
        else:
            self._image512 = None

        if image256:
            if isinstance(image256, ImageLocation):
                self._image256 = image256
            else:
                raise ValueError(Error.unacceptable_object_type)
        else:
            self._image256 = None

    def get_json_object(self):

        data = {
            "$type": MessageType.sticker_message,
            "stickerId": self._sticker_id,
            "stickerCollectionId": self._sticker_collection_id,
            "stickerCollectionAccessHash": self._sticker_collection_access_hash,
            "fastPreview": self._fast_preview,
            "image512": self._image512.get_json_object() if self._image512 else None,
            "image256": self._image256.get_json_object() if self._image256 else None,

        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        sticker_id = json_dict.get("stickerId", None)
        sticker_collection_id = json_dict.get("stickerCollectionId", None)
        sticker_collection_access_hash = json_dict.get("stickerCollectionAccessHash", None)
        fast_preview = json_dict.get("fastPreview ", None)
        image512 = ImageLocation.load_from_json(json_dict.get("image512", None)) if json_dict.get("image512",
                                                                                                  None) else None
        image256 = ImageLocation.load_from_json(json_dict.get("image256", None)) if json_dict.get("image256",
                                                                                                  None) else None

        return cls(sticker_id=sticker_id, sticker_collection_id=sticker_collection_id,
                   sticker_collection_access_hash=sticker_collection_access_hash, fast_preview=fast_preview,
                   image512=image512, image256=image256)



from balebot.models.messages.json_message import JsonMessage
from balebot.filters.filter import Filter


class JsonFilter(Filter):
    def match(self, message):
        return isinstance(message, JsonMessage)



import json as json_handler

from balebot.models.base_models.request_body import RequestBody
from balebot.models.constants.request_type import RequestType


class GetFileUploadUrl(RequestBody):
    def __init__(self, size, crc, file_type, is_server=False):
        self.size = int(size)
        self.crc = str(crc)
        self.is_server = bool(is_server)

        # it can be "photo" or "file"
        self.file_type = str(file_type)

    def get_json_object(self):
        data = {
            "$type": RequestType.get_file_upload_url,
            "size": self.size,
            "crc": self.crc,
            "isServer": self.is_server,
            'fileType': self.file_type,
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())



import json as json_handler

from balebot.models.messages.base_message import BaseMessage
from balebot.models.constants.errors import Error

from balebot.models.constants.message_type import MessageType


class UnsupportedMessage(BaseMessage):
    def get_json_object(self):
        data = {
            "$type": MessageType.unsupported_message,
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        return cls()



from balebot.filters import DefaultFilter
from balebot.filters.filter import Filter
from balebot.handlers.handler import Handler
from balebot.models.base_models.fat_seq_update import FatSeqUpdate


class MessageHandler(Handler):
    def __init__(self, filters, callback):
        super(MessageHandler, self).__init__(callback=callback)

        if not filters:
            filters = DefaultFilter()

        if isinstance(filters, list) and all(
                isinstance(message_filter, Filter) for message_filter in filters):
            self.filters = filters
        elif isinstance(filters, Filter):
            self.filters = [filters]
        else:
            raise ValueError("filters don't have acceptable format.")

    def check_update(self, update):
        if isinstance(update, FatSeqUpdate) and update.is_message_update():
            message = update.get_effective_message()

            return any(message_filter.match(message) for message_filter in self.filters)

    def handle_update(self, dispatcher, update):
        return self.callback(bot=dispatcher.bot, update=update)

    def is_default_handler(self):
        for message_filter in self.filters:
            if isinstance(message_filter, DefaultFilter):
                return True



class Filter:
    def __init__(self, validator=None):
        self.validator = validator if callable(validator) else None

    def match(self, message):
        raise NotImplementedError

    def validate(self, obj):
        return self.validator(obj) if self.validator else False



class Handler:
    def __init__(self, callback):
        self.callback = callback

    def check_update(self, update):
        raise NotImplementedError

    def handle_update(self, dispatcher, update):
        raise NotImplementedError

    def is_default_handler(self):
        raise NotImplementedError



class RequestType:
    get_file_download_url = "GetFileDownloadUrl"
    get_file_upload_url = "GetFileUploadUrl"

    create_group = "CreateGroup"
    get_group_api_struct = "GetGroupApiStruct"
    invite_user = "InviteUser"

    send_message = "SendMessage"
    edit_message = "EditMessage"

    get_difference = "GetDifference"
    get_last_seq = "GetLastSeq"



import json as json_handler

from balebot.models.base_models.group_peer import GroupPeer
from balebot.models.base_models.user_peer import UserPeer

from balebot.models.base_models.request_body import RequestBody
from balebot.models.constants.request_type import RequestType


class InviteUser(RequestBody):
    def __init__(self, group_peer, user_peer):
        if isinstance(group_peer, GroupPeer):
            self.group_peer = group_peer

        if isinstance(user_peer, UserPeer):
            self.user_peer = user_peer

    def get_json_object(self):
        data = {
            "$type": RequestType.invite_user,
            "groupPeer": self.group_peer.get_json_object(),
            "userPeer": self.user_peer.get_json_object(),
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())



import json as json_handler
from balebot.models.constants.errors import Error

from balebot.models.base_models.jsonable import Jsonable


class FileLocation(Jsonable):
    def __init__(self, file_id, access_hash, file_storage_version=1):
        self.file_id = str(file_id)
        self.access_hash = str(access_hash)
        self.file_storage_version = int(file_storage_version)

    def get_json_object(self):
        data = {
            "fileId": self.file_id,
            "accessHash": self.access_hash,
            "fileStorageVersion": self.file_storage_version
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        file_id = json_dict.get('fileId', None)
        access_hash = json_dict.get('accessHash', None)
        file_storage_version = json_dict.get('fileStorageVersion', None)

        if (not file_id) or (not access_hash):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(file_id=file_id, access_hash=access_hash, file_storage_version=file_storage_version)



import json as json_handler

from balebot.models.base_models.request_body import RequestBody
from balebot.models.constants.request_type import RequestType


class GetLastSequence(RequestBody):
    def __init__(self):
        self.seq = "input"

    def get_json_object(self):
        data = {
            "$type": RequestType.get_last_seq,
            "seq": self.seq
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())



from balebot.models.base_models.banking import deprecated_receipt
from balebot.models.base_models.banking import deprecated_receipt_message
from balebot.models.base_models.banking import receipt_message
from balebot.models.constants.bank_ext_types import BankExtTypes


class BankExtFactory:
    @staticmethod
    def create_bank_ext(json_dict):
        bank_ext_type = json_dict.get("$type", None)

        if bank_ext_type == BankExtTypes.deprecated_receipt_message:
            return deprecated_receipt_message.DeprecatedReceiptMessage.load_from_json(json_dict)

        elif bank_ext_type == BankExtTypes.deprecated_receipt:
            return deprecated_receipt.DeprecatedReceipt.load_from_json(json_dict)

        elif bank_ext_type == BankExtTypes.receipt_message:
            return receipt_message.ReceiptMessage.load_from_json(json_dict)

        else:
            return None



import json as json_handler
from balebot.models.constants.errors import Error

from balebot.models.base_models.group import Group
from balebot.models.base_models.user import User
from balebot.models.factories import update_body_factory
from balebot.models.server_updates.bot_read_update_body import BotReadUpdate
from balebot.models.server_updates.message_update_body import MessageUpdate
from balebot.models.server_updates.raw_update_body import RawUpdate

from balebot.models.server_updates.bot_received_update_body import BotReceivedUpdate


class FatSeqUpdate:
    def __init__(self, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        self.json_dict = json_dict
        self.seq = json_dict.get("seq", None)

        self.body = update_body_factory.UpdateBodyFactory.create_update_body(json_dict.get("body", None))

        temp_users = json_dict.get("users", None)
        self.users = [User.load_from_json(user[1]) for user in temp_users]

        temp_groups = json_dict.get("groups", None)
        self.groups = [Group.load_from_json(group[1]) for group in temp_groups]

    def is_message_update(self):
        return isinstance(self.body, MessageUpdate)

    def is_read_update(self):
        return isinstance(self.body, BotReadUpdate)

    def is_received_update(self):
        return isinstance(self.body, BotReceivedUpdate)

    def is_raw_update(self):
        return isinstance(self.body, RawUpdate)

    def get_effective_user(self):
        user = None
        if self.is_message_update():
            user = self.body.peer
        elif self.is_received_update():
            user = self.body.peer
        elif self.is_read_update():
            user = self.body.peer
        return user

    def get_effective_message(self):
        message = None
        if self.is_message_update():
            message = self.body.message
        return message

    def get_quoted_message(self):
        quoted_message = None
        if self.is_message_update():
            if self.body.quoted_message:
                quoted_message = self.body.quoted_message.message
        return quoted_message

    def get_quoted_sender_peer_id(self):
        user_id = None
        if self.is_message_update():
            if self.body.quoted_message:
                user_id = self.body.quoted_message.sender_id
        return user_id



import json as json_handler

from balebot.models.base_models.avatar import Avatar
from balebot.models.base_models.bot_command import BotCommand
from balebot.models.base_models.contact_record import ContactRecord
from balebot.models.constants.errors import Error

from balebot.models.base_models.jsonable import Jsonable


class User(Jsonable):
    def __init__(self, user_id, access_hash, name, contact_records, preferred_languages, bot_commands,
                 sex=None, about=None, avatar=None, username=None, is_bot=None, time_zone=None):
        self.id = str(user_id)
        self.access_hash = str(access_hash)
        self.name = str(name)

        if all(isinstance(contact_record, ContactRecord) for contact_record in contact_records):
            self.contact_records = [ContactRecord(contact_record) for contact_record in contact_records]
        else:
            ValueError(Error.unacceptable_object_type)

        self.preferred_languages = list(preferred_languages)

        if all(isinstance(bot_command, BotCommand) for bot_command in bot_commands):
            self.bot_commands = bot_commands
        else:
            ValueError(Error.unacceptable_object_type)

        self.sex = int(sex) if sex else None
        self.about = str(about) if about else None

        if avatar:
            if isinstance(avatar, Avatar):
                self.avatar = avatar
            else:
                raise ValueError(Error.unacceptable_object_type)
        else:
            self.avatar = None

        self.username = str(username) if username else None
        self.is_bot = bool(is_bot) if is_bot else None
        self.time_zone = str(time_zone) if time_zone else None

    def get_json_object(self):

        data = {
            "id": self.id,
            "accessHash": self.access_hash,
            "name": self.name,
            "contactRecords": self.contact_records,
            "preferredLanguages": self.preferred_languages,
            "botCommands": self.bot_commands,
            "sex": self.sex,
            "about": self.about,
            "avatar": self.avatar,
            "username": self.username,
            "isBot": self.is_bot,
            "timeZone": self.time_zone,

        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        user_id = json_dict.get('id', None)
        access_hash = json_dict.get('accessHash', None)
        name = json_dict.get('name', None)
        contact_records = [ContactRecord(contact_record) for contact_record in json_dict.get('contactRecords', None)]
        preferred_languages = list(json_dict.get('preferredLanguages', None))
        bot_commands = [BotCommand.load_from_json(bot_command) for bot_command in json_dict.get('botCommands', None)]

        if (not user_id) or (not access_hash) or (name is None) or (contact_records is None) or (
                    preferred_languages is None) or (bot_commands is None):
            raise ValueError(Error.none_or_invalid_attribute)

        sex = json_dict.get('sex', None)
        about = json_dict.get('about', None)
        avatar = Avatar.load_from_json(json_dict.get('avatar', None)) if json_dict.get('avatar', None) else None
        username = json_dict.get('username', None)
        is_bot = json_dict.get('is_bot', None)
        time_zone = json_dict.get('timeZone', None)

        return cls(user_id=user_id, access_hash=access_hash, contact_records=contact_records,
                   preferred_languages=preferred_languages, bot_commands=bot_commands, name=name, sex=sex,
                   about=about, avatar=avatar, username=username, is_bot=is_bot, time_zone=time_zone)



import json as json_handler
from balebot.models.constants.errors import Error


class ContactRecord:
    def __init__(self, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        self.type = json_dict.get("$type", None)

        if self.type == "Email":
            self.email = json_dict.get("email", None)

        elif self.type == "Phone":
            self.phone = json_dict.get("phone", None)



from balebot.models.messages.photo_message import PhotoMessage
from balebot.filters.filter import Filter


class PhotoFilter(Filter):
    def match(self, message):
        if isinstance(message, PhotoMessage):
            return self.validator(message) if self.validator else True
        else:
            return False



import json as json_handler

from balebot.models.base_models.request_body import RequestBody
from balebot.models.constants.errors import Error

from balebot.models.base_models.jsonable import Jsonable


class Request(Jsonable):
    id_counter = 0

    def __init__(self, service, body):
        self._id = str(Request.id_counter)
        self.service = str(service)
        if isinstance(body, RequestBody):
            self.body = body
        else:
            raise ValueError(Error.unacceptable_object_type)

        Request.id_counter += 1

    def get_json_object(self):
        data = {
            "$type": "Request",
            "id": self._id,
            "service": self.service,
            "body": self.body.get_json_object(),
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        pass

    @property
    def id(self):
        return self._id



class Jsonable:
    def get_json_object(self):
        raise NotImplementedError

    def get_json_str(self):
        raise NotImplementedError

    @classmethod
    def load_from_json(cls, json):
        raise NotImplementedError



from balebot.models.constants.update_body_type import UpdateBodyType

from balebot.models.server_updates import bot_read_update_body
from balebot.models.server_updates import message_update_body
from balebot.models.server_updates import raw_update_body
from balebot.models.server_updates import bot_received_update_body


class UpdateBodyFactory:
    @staticmethod
    def create_update_body(body_json_dict):
        update_body_type = body_json_dict.get("$type", None)

        if update_body_type == UpdateBodyType.message:
            return message_update_body.MessageUpdate(body_json_dict)

        elif update_body_type == UpdateBodyType.bot_read_update:
            return bot_read_update_body.BotReadUpdate(body_json_dict)

        elif update_body_type == UpdateBodyType.bot_received_update:
            return bot_received_update_body.BotReceivedUpdate(body_json_dict)

        elif update_body_type == UpdateBodyType.raw_update:
            return raw_update_body.RawUpdate(body_json_dict)



class ValueType:
    string_val = "StringVal"
    int32_val = "Int32Val"
    int64_val = "Int64Val"
    double_val = "DoubleVal"
    array_val = "ArrayVal"
    map_value = "MapValue"
    boolean_value = "BooleanValue"



import functools
import asyncio
import os

import aiohttp
import io
import sys
from PIL import Image
from balebot.models.base_models import response
from balebot.connection.network import Network
from balebot.bale_future import BaleFuture
from balebot.models.client_requests import *

from balebot.models.base_models import BotQuotedMessage, FatSeqUpdate, GroupPeer, Request, UserPeer
from balebot.models.client_requests.sequence.get_last_sequence import GetLastSequence
from balebot.models.constants.request_to_response_mapper import RequestToResponseMapper
from balebot.models.constants.service_type import ServiceType
from balebot.models.messages import BaseMessage, TextMessage, PhotoMessage, DocumentMessage
from balebot.utils.util_functions import get_file_crc32, get_file_size, get_file_buffer, get_image_thumbnails


class Bot:
    def __init__(self, loop, token, incoming_queue, outgoing_queue, bale_futures, timeout):
        self._loop = loop
        self.network = Network(token=token,
                               incoming_queue=incoming_queue,
                               outgoing_queue=outgoing_queue,
                               loop=loop)

        self._bale_futures = bale_futures
        self.timeout = timeout

    def set_future(self, request_id, request_body, success_callback=None, failure_callback=None, **kwargs):
        response_body_module, response_body_class = RequestToResponseMapper.get_response(request_body)
        bale_future = BaleFuture(request_id, response_body_module, response_body_class,
                                 success_callback, failure_callback, **kwargs)
        self._bale_futures.append(bale_future)

        self._loop.call_later(self.timeout, functools.partial(self.timeout_future, bale_future))

    def timeout_future(self, bale_future):
        if bale_future in self._bale_futures:
            bot_timeout_json = {"body": {"tag": "CLIENT_REQUEST_TIMEOUT"}}
            error_response = response.Response(bot_timeout_json)
            bale_future.reject(error_response)
            self._bale_futures.remove(bale_future)

    def send_request(self, request_data):
        self.network.send(request_data)

    def reply(self, update, message, success_callback=None, failure_callback=None, **kwargs):
        if isinstance(update, FatSeqUpdate) and update.is_message_update():
            message_id = update.body.random_id
            user_peer = update.get_effective_user()
            if isinstance(message, BaseMessage):
                return self.send_message(message, user_peer,
                                         BotQuotedMessage(message_id, user_peer),
                                         success_callback=success_callback,
                                         failure_callback=failure_callback,
                                         **kwargs)
            else:
                return self.send_message(TextMessage(message), user_peer,
                                         BotQuotedMessage(message_id, user_peer),
                                         success_callback=success_callback,
                                         failure_callback=failure_callback,
                                         **kwargs)

    def respond(self, update, message, success_callback=None, failure_callback=None, **kwargs):
        user_peer = update.get_effective_user()
        if isinstance(message, BaseMessage):
            return self.send_message(message, user_peer,
                                     success_callback=success_callback,
                                     failure_callback=failure_callback,
                                     **kwargs)
        else:
            return self.send_message(TextMessage(message), user_peer,
                                     success_callback=success_callback,
                                     failure_callback=failure_callback,
                                     **kwargs)

    # messaging
    def send_message(self, message, peer, quoted_message=None, random_id=None, success_callback=None,
                     failure_callback=None, **kwargs):
        receiver = peer
        request_body = SendMessage(message=message, receiver_peer=receiver,
                                   quoted_message=quoted_message, random_id=random_id)
        request = Request(service=ServiceType.Messaging, body=request_body)
        self.set_future(request.id, request_body, success_callback, failure_callback, **kwargs)
        self.send_request(request.get_json_str())
        return request

    def edit_message(self, message, user_peer, random_id, success_callback=None, failure_callback=None,
                     **kwargs):
        text_message = message
        receiver = user_peer
        request_body = EditMessage(updated_message=text_message, peer=receiver, random_id=random_id)
        request = Request(service=ServiceType.Messaging, body=request_body)
        self.set_future(request.id, request_body, success_callback, failure_callback, **kwargs)
        self.send_request(request.get_json_str())
        return request

    # group
    def create_group(self, title, success_callback=None, failure_callback=None, **kwargs):
        request_body = CreateGroup(title)
        request = Request(service=ServiceType.Groups, body=request_body)
        self.set_future(request.id, request_body, success_callback, failure_callback, **kwargs)
        self.send_request(request.get_json_str())
        return request

    def get_group_api_struct(self, group_id, client_user_id, success_callback=None, failure_callback=None, **kwargs):
        request_body = GetGroupApiStruct(group_id=group_id, client_user_id=client_user_id)
        request = Request(service=ServiceType.Groups, body=request_body)
        self.set_future(request.id, request_body, success_callback, failure_callback, **kwargs)
        self.send_request(request.get_json_str())
        return request

    def invite_user(self, group_peer_id, group_peer_access_hash, user_peer_id,
                    user_peer_access_hash, success_callback=None, failure_callback=None, **kwargs):
        group_peer = GroupPeer(peer_id=group_peer_id, access_hash=group_peer_access_hash)
        user_peer = UserPeer(peer_id=user_peer_id, access_hash=user_peer_access_hash)
        request_body = InviteUser(group_peer=group_peer, user_peer=user_peer)
        request = Request(service=ServiceType.Groups, body=request_body)
        self.set_future(request.id, request_body, success_callback, failure_callback, **kwargs)
        self.send_request(request.get_json_str())
        return request

    # sequence-update
    def get_difference(self, seq, how_many, success_callback=None, failure_callback=None, **kwargs):
        request_body = GetDifference(seq=seq, how_many=how_many)
        request = Request(service=ServiceType.SequenceUpdate, body=request_body)
        self.set_future(request.id, request_body, success_callback, failure_callback, **kwargs)
        self.send_request(request.get_json_str())
        return request

    # last_sequence
    def get_last_seq(self, success_callback=None, failure_callback=None, **kwargs):
        request_body = GetLastSequence()
        request = Request(service=ServiceType.SequenceUpdate, body=request_body)
        self.set_future(request.id, request_body, success_callback, failure_callback, **kwargs)
        self.send_request(request.get_json_str())
        return request

    # file
    def get_file_download_url(self, file_id, user_id, file_type, file_version=1, is_server=False,
                              is_resume_upload=False, success_callback=None, failure_callback=None, **kwargs):
        request_body = GetFileDownloadUrl(file_id, user_id, file_type, file_version, is_server, is_resume_upload)
        request = Request(service=ServiceType.Files, body=request_body)
        self.set_future(request.id, request_body, success_callback, failure_callback, **kwargs)
        self.send_request(request.get_json_str())
        return request

    def get_file_upload_url(self, size, crc, file_type, is_server=False,
                            success_callback=None, failure_callback=None, **kwargs):
        request_body = GetFileUploadUrl(size, crc, file_type, is_server)
        request = Request(service=ServiceType.Files, body=request_body)
        self.set_future(request.id, request_body, success_callback, failure_callback, **kwargs)
        self.send_request(request.get_json_str())
        return request

    def download_file(self, file_id, user_id, file_type="file", success_callback=None, failure_callback=None, **kwargs):

        future = BaleFuture(request_id=None, response_body_module=None, response_body_class=None,
                            success_callback=success_callback, failure_callback=failure_callback, **kwargs)

        def file_download_url_success(result, user_data):
            async def get_data(download_url):
                try:
                    async with aiohttp.ClientSession() as session:
                        async with session.get(download_url) as download_response:
                            status = download_response.status

                            if status == 200:
                                byte_stream = await download_response.content.read()
                                future.set_user_data(byte_stream=byte_stream)
                                future.resolve(response=None)
                            else:
                                future.reject(response=None)
                except Exception as e:
                    future.reject(response=None)
            url = result.body.url
            asyncio.ensure_future(get_data(url))

        def file_download_url_failure(result, user_data):
            future.reject(response=result)

        self.get_file_download_url(file_id, user_id, file_type,
                                   success_callback=file_download_url_success,
                                   failure_callback=file_download_url_failure)

    def upload_file(self, file, file_type, success_callback=None, failure_callback=None, **kwargs):

        future = BaleFuture(request_id=None, response_body_module=None, response_body_class=None,
                            success_callback=success_callback, failure_callback=failure_callback, **kwargs)

        buffer = get_file_buffer(file=file)
        if buffer is None:
            future.reject(response=None)
            return

        file_size = get_file_size(buffer)
        file_crc32 = get_file_crc32(buffer)

        def file_upload_url_success(result, user_data):
            file_id = result.body.file_id
            user_id = result.body.user_id
            url = result.body.url
            dup = result.body.dup

            data = buffer
            headers = {'filesize': str(file_size)}

            async def upload_data():
                try:
                    async with aiohttp.ClientSession() as session:
                        async with session.put(url, data=data, headers=headers) as upload_response:
                            status = upload_response.status
                            if status == 200:
                                future.set_user_data(file_id=file_id, user_id=user_id, url=url, dup=dup)
                                future.resolve(response=None)
                            else:
                                future.reject(response=None)
                except Exception as e:
                    future.reject(response=None)
            asyncio.ensure_future(upload_data())

        def file_upload_url_failure(result, user_data):
            future.reject(response=result)

        self.get_file_upload_url(size=file_size, crc=file_crc32, file_type=file_type,
                                 success_callback=file_upload_url_success,
                                 failure_callback=file_upload_url_failure)

    def send_photo(self, user_peer, image, caption_text="", name="", file_storage_version=1, mime_type="image/jpeg",
                   success_callback=None, failure_callback=None, **kwargs):
        image_buffer = get_file_buffer(file=image)
        file_size = sys.getsizeof(image_buffer)
        im = Image.open(io.BytesIO(image_buffer))
        width, height = im.size
        thumb = get_image_thumbnails(im)

        def success_upload_image(user_data, server_response):
            file_id = str(server_response.get("file_id", None))
            access_hash = str(server_response.get("user_id", None))
            photo_message = PhotoMessage(file_id=file_id, access_hash=access_hash, name=name, file_size=file_size,
                                         mime_type=mime_type, file_storage_version=file_storage_version, width=width,
                                         height=height, caption_text=TextMessage(text=caption_text), thumb=thumb)
            self.send_message(message=photo_message, peer=user_peer, success_callback=success_callback,
                              failure_callback=failure_callback, kwargs=kwargs)

        self.upload_file(file=image, file_type="file", success_callback=success_upload_image,
                         failure_callback=failure_callback)

    def send_document(self, user_peer, doc_file, mime_type, caption_text="", file_type="file", name="",
                      file_storage_version=1, success_callback=None, failure_callback=None, **kwargs):
        file_size = os.path.getsize(doc_file)

        def success_upload_document(user_data, server_response):
            file_id = str(server_response.get("file_id", None))
            access_hash = str(server_response.get("user_id", None))
            document_message = DocumentMessage(file_id=file_id, access_hash=access_hash, name=name,
                                               file_size=file_size, mime_type=mime_type,
                                               caption_text=TextMessage(text=caption_text),
                                               file_storage_version=file_storage_version)
            self.send_message(message=document_message, peer=user_peer, success_callback=success_callback,
                              failure_callback=failure_callback, kwargs=kwargs)

        self.upload_file(file=doc_file, file_type=file_type, success_callback=success_upload_document,
                         failure_callback=failure_callback)



from balebot.models.base_models import fat_seq_update

from balebot.models.base_models import response


class ServerUpdateFactory:
    @staticmethod
    def create_update(json_dict):
        update_type = json_dict.get("$type", None)

        if update_type == "Response":
            return response.Response(json_dict)

        elif update_type == "FatSeqUpdate":
            return fat_seq_update.FatSeqUpdate(json_dict)

        else:
            return



import json as json_handler
from balebot.models.constants.errors import Error

from balebot.models.base_models.jsonable import Jsonable


class BotCommand(Jsonable):
    def __init__(self, slash_command, description, loc_key="a"):
        self.slash_command = str(slash_command)
        self.description = str(description)
        self.loc_key = str(loc_key) if loc_key else None

    def get_json_object(self):
        data = {
            "slashCommand": self.slash_command,
            "description": self.description,
            "locKey": self.loc_key,

        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        slash_command = json_dict.get('slashCommand', None)
        description = json_dict.get('description', None)
        loc_key = json_dict.get('locKey', None)

        if (slash_command is None) or (description is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(slash_command=slash_command, description=description, loc_key=loc_key)



import unittest
from unittest.mock import MagicMock

from balebot.models.server_responses.messaging.message_edited import MessageEdited


class TestMessageEdited(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.edit_message_response_body = '{"seq":"2090","date":"1536053835915"}'

    def test_message_edited(self):
        message_edited_body = MessageEdited(self.edit_message_response_body)
        self.assertEqual(message_edited_body.date, '1536053835915')
        self.assertEqual(message_edited_body.seq, '2090')



from asyncio import Future
import collections


class BaleFuture:
    def __init__(self, request_id, response_body_module, response_body_class,
                 success_callback=None, failure_callback=None, **kwargs):

        self.request_id = str(request_id)
        self.response_body_module = response_body_module
        self.response_body_class = response_body_class
        self.success_callback = success_callback if isinstance(success_callback, collections.Callable) else None
        self.failure_callback = failure_callback if isinstance(failure_callback, collections.Callable) else None
        self.user_data = kwargs if kwargs else {}

    def set_user_data(self, **kwargs):
        if kwargs:
            self.user_data.update(kwargs)

    def resolve(self, response):
        if self.success_callback:
            self.success_callback(response, self.user_data)

    def reject(self, response):
        if self.failure_callback:
            self.failure_callback(response, self.user_data)



import logging
import graypy

from balebot.config import Config


class Logger:
    logger = None

    @staticmethod
    def init_logger():

        use_graylog = Config.use_graylog
        source = Config.source
        graylog_host = Config.graylog_host
        graylog_port = Config.graylog_port
        log_level = Config.log_level
        log_facility_name = Config.log_facility_name

        temp_logger = logging.getLogger(log_facility_name)
        temp_logger.setLevel(log_level)

        log_handlers = []

        if use_graylog == "0":
            handler = logging.StreamHandler()
            formatter = logging.Formatter(
                '%(asctime)s  %(filename)s:%(lineno)d  %(levelname)s:\n"%(message)s"'
            )
            handler.setFormatter(formatter)
            log_handlers.append(handler)

        elif use_graylog == "1" and graylog_host and source and graylog_port is not None \
                and isinstance(graylog_port, int):
            log_handlers.append(graypy.GELFHandler(host=graylog_host, port=graylog_port, localname=source))

        elif use_graylog == "2" and graylog_host and source and graylog_port is not None \
                and isinstance(graylog_port, int):
            handler1 = graypy.GELFHandler(host=graylog_host, port=graylog_port, localname=source)

            handler2 = logging.StreamHandler()
            formatter = logging.Formatter(
                '%(asctime)s  %(filename)s:%(lineno)d  %(levelname)s:\n"%(message)s"'
            )
            handler2.setFormatter(formatter)

            log_handlers.append(handler1)
            log_handlers.append(handler2)

        for log_handler in log_handlers:
            temp_logger.addHandler(log_handler)

        Logger.logger = temp_logger
        return Logger.logger

    @staticmethod
    def get_logger():
        if Logger.logger:
            return Logger.logger
        else:
            return Logger.init_logger()



import json as json_handler

from balebot.models.base_models.value_types.raw_value import RawValue
from balebot.models.constants.errors import Error

from balebot.models.factories import raw_value_factory


class MapValueItem:
    def __init__(self, key, value):
        if isinstance(value, RawValue):
            self.value = value
        else:
            raise ValueError(Error.unacceptable_object_type)
        self.key = str(key)

    def get_json_object(self):

        data = {
            "key": self.key,
            "value": self.value.get_json_object(),
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        key = json_dict.get('key', None)

        tmp_value = json_dict.get('value', None)
        value = raw_value_factory.RawValueFactory.create_raw_value(tmp_value)

        if (key is None) or (value is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(key=key, value=value)



from balebot.models.client_requests.files.get_file_download_url import GetFileDownloadUrl
from balebot.models.client_requests.files.get_file_upload_url import GetFileUploadUrl

from balebot.models.client_requests.group.create_group import CreateGroup
from balebot.models.client_requests.group.get_group_api_struct import GetGroupApiStruct
from balebot.models.client_requests.group.invite_user import InviteUser

from balebot.models.client_requests.messaging.send_message import SendMessage
from balebot.models.client_requests.messaging.edit_message import EditMessage

from balebot.models.client_requests.sequence.get_difference import GetDifference



import math
import random
import binascii
from io import BufferedReader
import re


def generate_random_id():
    return str(math.floor(random.random() * 1800000)) + str(math.floor(random.random() * 4000000)) + str(
        math.floor(random.random() * 55000))


def get_file_buffer(file, file_operation_mode="rb"):
    if isinstance(file, str):
        opened_file = open(file, file_operation_mode)
        buffer = opened_file.read()
        opened_file.close()
    elif isinstance(file, BufferedReader):
        buffer = file.read()
        file.close()
    elif isinstance(file, bytes):
        buffer = file
    else:
        return None

    return buffer


def get_file_size(file):
    buffer = get_file_buffer(file=file)
    return len(buffer)


def get_file_crc32(file):
    buffer = get_file_buffer(file=file)
    crc_buffer = (binascii.crc32(buffer) & 0xFFFFFFFF)
    return crc_buffer


def get_image_thumbnails(im):
    from io import BytesIO
    import base64
    from PIL import Image
    size = 80, 80
    im.thumbnail(size, Image.ANTIALIAS)
    output = BytesIO()
    im.save(output, format='JPEG')
    im_data = output.getvalue()
    thumb = '{}'.format(base64.b64encode(im_data).decode())
    return thumb


def arabic_to_eng_number(number):
    number = str(number)
    return number.translate(str.maketrans('۰۱۲۳۴۵۶۷۸۹٠١٢٣٤٥٦٧٨٩', '01234567890123456789'))


def eng_to_arabic_number(number):
    number = str(number)
    return number.translate(str.maketrans('0123456789', '۰۱۲۳۴۵۶۷۸۹'))


def phone_number_validation(phone_num):
    if re.match(r'^(\+98|0098|0)?9\d{9}$', phone_num):
        return True
    else:
        return False


def standardize_phone_number(number):
    number_str = str(number)
    if number_str.startswith("0098"):
        return "+98" + number_str[4:]
    elif number_str.startswith("0"):
        return "+98" + number_str[1:]



from balebot.models.messages.sticker_message import StickerMessage
from balebot.filters.filter import Filter


class StickerFilter(Filter):
    def match(self, message):
        return isinstance(message, StickerMessage)



import json as json_handler

from balebot.models.base_models.jsonable import Jsonable
from balebot.models.constants.errors import Error


class Peer(Jsonable):
    def __init__(self, peer_type, peer_id, access_hash):
        self.type = str(peer_type)
        self.peer_id = str(peer_id)
        self.access_hash = str(access_hash)

    def get_json_object(self):
        data = {
            "$type": self.type,
            "id": self.peer_id,
            "accessHash": self.access_hash,
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        peer_type = json_dict.get('$type', None)
        peer_id = json_dict.get('id', None)
        access_hash = json_dict.get('accessHash', None)

        if (not peer_type) or (not peer_id) or (not access_hash):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(peer_type=peer_type, peer_id=peer_id, access_hash=access_hash)



import json as json_handler
from balebot.models.constants.errors import Error

from balebot.models.factories import message_factory


class BotApiQuotedMessage:
    def __init__(self, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        self.message_id = json_dict.get("messageId", None)
        self.public_group_id = json_dict.get("publicGroupId", None)
        self.sender_id = json_dict.get("senderId", None)
        self.messageDate = json_dict.get("messageDate", None)
        self.message = message_factory.MessageFactory.create_message(json_dict.get("message"))



import json as json_handler

from balebot.models.base_models.request_body import RequestBody
from balebot.models.constants.request_type import RequestType


class GetFileDownloadUrl(RequestBody):
    def __init__(self, file_id, user_id, file_type, file_version=1, is_server=False, is_resume_upload=False):
        self.file_id = str(file_id)
        self.user_id = str(user_id)
        self.file_version = int(file_version)
        self.is_server = bool(is_server)
        self.is_resume_upload = bool(is_resume_upload)

        # it can be "photo" or "file"
        self.file_type = str(file_type)

    def get_json_object(self):
        data = {
            "$type": RequestType.get_file_download_url,
            "fileId": self.file_id,
            "userId": self.user_id,
            "fileVersion": self.file_version,
            "isServer": self.is_server,
            "isResumeUpload": self.is_resume_upload,
            'fileType': self.file_type,
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())



from balebot.models.constants.value_type import ValueType
from balebot.models.base_models.value_types import int32_val
from balebot.models.base_models.value_types import int64_val
from balebot.models.base_models.value_types import boolean_value
from balebot.models.base_models.value_types import double_val
from balebot.models.base_models.value_types import string_val
from balebot.models.base_models.value_types import array_val
from balebot.models.base_models.value_types import map_value


class RawValueFactory:
    @staticmethod
    def create_raw_value(json_dict):
        raw_value_type = json_dict.get("$type", None)

        if raw_value_type == ValueType.int32_val:
            return int32_val.Int32Val.load_from_json(json_dict)

        if raw_value_type == ValueType.int64_val:
            return int64_val.Int64Val.load_from_json(json_dict)

        elif raw_value_type == ValueType.boolean_value:
            return boolean_value.BooleanValue.load_from_json(json_dict)

        elif raw_value_type == ValueType.double_val:
            return double_val.DoubleVal.load_from_json(json_dict)

        elif raw_value_type == ValueType.string_val:
            return string_val.StringVal.load_from_json(json_dict)

        elif raw_value_type == ValueType.array_val:
            return array_val.ArrayVal.load_from_json(json_dict)

        elif raw_value_type == ValueType.map_value:
            return map_value.MapValue.load_from_json(json_dict)

    @staticmethod
    def get_value(raw_value):
        if isinstance(raw_value, int32_val.Int32Val):
            return raw_value.value

        if isinstance(raw_value, int64_val.Int64Val):
            return raw_value.value

        elif isinstance(raw_value, boolean_value.BooleanValue):
            return raw_value.value

        elif isinstance(raw_value, double_val.DoubleVal):
            return raw_value.value

        elif isinstance(raw_value, string_val.StringVal):
            return raw_value.text

        elif isinstance(raw_value, array_val.ArrayVal):
            return raw_value.values

        elif isinstance(raw_value, map_value.MapValue):
            return raw_value.items



import json as json_handler

from balebot.models.base_models.raw_json import RawJson
from balebot.models.messages.base_message import BaseMessage
from balebot.models.constants.errors import Error
from balebot.models.factories import raw_json_factory
from balebot.models.constants.message_type import MessageType


class JsonMessage(BaseMessage):
    def __init__(self, raw_json):
        if isinstance(raw_json, RawJson):
            self.raw_json = raw_json

    def get_json_object(self):

        data = {
            "$type": MessageType.json_message,
            "rawJson": self.raw_json.get_json_str()

        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        raw_json_dict = json_handler.loads(json_dict.get('rawJson', None))
        raw_json = raw_json_factory.RawJsonFactory.create_raw_json(raw_json_dict)

        if raw_json is None:
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(raw_json=raw_json)

    @staticmethod
    def is_raw_location_message(json):
        json_dict = json_handler.loads(json.get('rawJson'))
        raw_json_type = json_dict.get('dataType') if json_dict else None

        return raw_json_type == MessageType.location_message

    @staticmethod
    def is_raw_contact_message(json):
        json_dict = json_handler.loads(json.get('rawJson'))
        raw_json_type = json_dict.get('dataType') if json_dict else None

        return raw_json_type == MessageType.contact_message



import asyncio
import json as json_handler
import time
import traceback
from pathlib import Path

from balebot.bot import Bot
from balebot.config import Config
from balebot.filters import DefaultFilter, TextFilter
from balebot.handlers import Handler, CommandHandler, MessageHandler, QuotedMessageHandler
from balebot.models.base_models import Response
from balebot.models.base_models.fat_seq_update import FatSeqUpdate
from balebot.models.factories import server_update_factory
from balebot.models.messages import TextMessage
from balebot.utils.logger import Logger


class Dispatcher:
    def __init__(self, loop, token, bale_futures):

        self.logger = Logger.get_logger()
        self.incoming_queue = asyncio.Queue()
        self.outgoing_queue = asyncio.Queue()
        self.timeout = Config.request_timeout
        self.token = token
        self.bot = Bot(loop=loop,
                       token=token,
                       incoming_queue=self.incoming_queue,
                       outgoing_queue=self.outgoing_queue,
                       bale_futures=bale_futures,
                       timeout=self.timeout)

        self._bale_futures = bale_futures
        self.message_handlers = []
        self.error_handlers = []
        self.read_handler_object = None
        self.default_handler_object = None

        self.conversation_next_step_handlers = {}
        self.conversation_data = {}

        self.running = False

        self.is_batch_updates_processed = True
        self.last_seq = None
        self.real_time_fetch_updates = Config.real_time_fetch_updates
        self.continue_last_processed_seq = Config.continue_last_processed_seq
        self.timeInterval = Config.timeInterval
        self.last_poll_request_time = 0
        self.updates_number = Config.updates_number

        @self.message_handler(TextFilter(pattern=r"{}*".format(Config.monitoring_hash)))
        def handle_monitoring_msg(bot, update):
            monitoring_message = update.get_effective_message()
            monitoring_text = monitoring_message.text
            result_text = str(monitoring_text.split(Config.monitoring_hash)[1])
            result_message = TextMessage(text=result_text)
            bot.respond(update=update, message=result_message)

    async def run(self):

        self.running = True
        self.poll()
        while self.running:
            try:
                update_message = await self.incoming_queue.get()
                update_message_data = update_message.data
                update_message_json = json_handler.loads(update_message_data)
                self.process_update(update_message_json)

            except Exception as ex:
                self.logger.error(ex, extra={"tag": "err"})

                traceback.print_exc()
                bot_timeout_json = {"body": {"tag": "CLIENT_INTERNAL_ERROR"}}
                error_response = Response(bot_timeout_json)
                self.dispatch_error(error_response, ex)

    def stop(self):
        self.running = False

    def process_update(self, update_json):
        if update_json.get("body").get("peer") and update_json.get("body").get("peer").get("$type") == "Group" \
                and Config.group_shield == 1:
            return
        update = server_update_factory.ServerUpdateFactory.create_update(update_json)

        if isinstance(update, Response):
            response_id = update.id
            bale_future = self.find_future(response_id)

            if not bale_future:
                return

            update.create_body(bale_future.response_body_module, bale_future.response_body_class)
            if update.is_bot_error():
                bale_future.reject(update)
                return
            bale_future.resolve(update)
            return

        elif isinstance(update, FatSeqUpdate) and self.real_time_fetch_updates:
            self.process_fat_seq_update(update)

    def process_fat_seq_update(self, update):
        if update.is_message_update():
            user_peer = update.get_effective_user()
            user_id = user_peer.peer_id

            message_handled = False

            if (user_id in self.conversation_next_step_handlers) and self.conversation_next_step_handlers.get(
                    user_id, None):

                default_conversation_handler = None

                for handler in self.conversation_next_step_handlers.get(user_id, None):

                    if not handler.is_default_handler():
                        try:
                            if handler.check_update(update):
                                handler.handle_update(self, update)
                                message_handled = True
                                break
                        except Exception as ex:
                            self.logger.error(ex, extra={"tag": "err"})
                            traceback.print_exc()

                            try:
                                bot_timeout_json = {"body": {"tag": "CLIENT_INTERNAL_ERROR"}}
                                error_response = Response(bot_timeout_json)
                                self.dispatch_error(error_response, ex)
                            except Exception as ex:
                                self.logger.error(ex, extra={"tag": "err"})
                                traceback.print_exc()
                    else:
                        default_conversation_handler = handler

                if (not message_handled) and default_conversation_handler:
                    if default_conversation_handler.check_update(update):
                        default_conversation_handler.handle_update(self, update)

            else:
                for handler in self.message_handlers:
                    try:
                        if handler.check_update(update):
                            handler.handle_update(self, update)
                            message_handled = True
                            break
                    except Exception as ex:
                        self.logger.error(ex, extra={"tag": "err"})
                        traceback.print_exc()

                        try:
                            bot_timeout_json = {"body": {"tag": "CLIENT_INTERNAL_ERROR"}}
                            error_response = Response(bot_timeout_json)
                            self.dispatch_error(error_response, ex)
                        except Exception as ex:
                            self.logger.error(ex, extra={"tag": "err"})
                            traceback.print_exc()

                if (not message_handled) and self.default_handler_object:
                    if self.default_handler_object.check_update(update):
                        self.default_handler_object.handle_update(self, update)
            return

        elif update.is_read_update():
            if self.read_handler_object:
                try:
                    if self.read_handler_object.check_update(update):
                        self.read_handler_object.handle_update(self, update)
                except Exception as ex:
                    self.logger.error(ex, extra={"tag": "err"})
                    traceback.print_exc()

                    try:
                        bot_timeout_json = {"body": {"tag": "CLIENT_INTERNAL_ERROR"}}
                        error_response = Response(bot_timeout_json)
                        self.dispatch_error(error_response, ex)
                    except Exception as ex:
                        self.logger.error(ex, extra={"tag": "err"})
                        traceback.print_exc()

    def add_handler(self, handler):
        if isinstance(handler, Handler):
            if handler.is_default_handler():
                self.default_handler_object = handler
            else:
                self.message_handlers.append(handler)

    def add_handlers(self, handlers):
        if isinstance(handlers, list):
            for handler in handlers:
                self.add_handler(handler)
        else:
            self.add_handler(handlers)

    def remove_handler(self, handler):
        self.message_handlers.remove(handler)

    def add_error_handler(self, handler):
        self.error_handlers.append(handler)

    def add_error_handlers(self, handlers):
        if isinstance(handlers, list):
            self.error_handlers += handlers
        else:
            self.error_handlers.append(handlers)

    def remove_error_handler(self, handler):
        self.error_handlers.remove(handler)

    def dispatch_error(self, update, error):
        for error_handler in self.error_handlers:
            error_handler(self.bot, update, error)

    def find_future(self, response_id):
        for i in range(len(self._bale_futures)):
            if self._bale_futures[i].request_id == response_id:
                return self._bale_futures.pop(i)

    def message_handler(self, filters):
        def decorator(callback_func):
            handler = MessageHandler(filters, callback_func)
            self.add_handler(handler)

            return callback_func

        return decorator

    def quoted_message_handler(self, filters):
        def decorator(callback_func):
            handler = QuotedMessageHandler(filters, callback_func)
            self.add_handler(handler)

            return callback_func
        return decorator

    def command_handler(self, commands, include_template_response=False):
        def decorator(callback_func):
            handler = CommandHandler(commands, callback_func, include_template_response)
            self.add_handler(handler)

            return callback_func

        return decorator

    def default_handler(self):
        def decorator(callback_func):
            handler = MessageHandler(DefaultFilter(), callback_func)
            self.add_handler(handler)

            return callback_func

        return decorator

    def error_handler(self):
        def decorator(callback_func):
            self.add_error_handler(callback_func)

            return callback_func

        return decorator

    def register_conversation_next_step_handler(self, update, handlers):
        if isinstance(update, FatSeqUpdate) and update.is_message_update():

            next_step_handlers = []

            if isinstance(handlers, list):
                for handler in handlers:
                    if isinstance(handler, Handler):
                        next_step_handlers.append(handler)
            elif isinstance(handlers, Handler):
                next_step_handlers.append(handlers)

            user_peer = update.get_effective_user()
            user_id = user_peer.peer_id

            if user_id in self.conversation_next_step_handlers:
                self.conversation_next_step_handlers[user_id].clear()
                self.conversation_next_step_handlers[user_id] += next_step_handlers
            else:
                self.conversation_next_step_handlers[user_id] = next_step_handlers

    def conversation_finished(self, update):
        if isinstance(update, FatSeqUpdate) and update.is_message_update():
            user_peer = update.get_effective_user()
            user_id = user_peer.peer_id
            return user_id not in self.conversation_next_step_handlers and user_id not in self.conversation_data

    def finish_conversation(self, update):
        if isinstance(update, FatSeqUpdate) and update.is_message_update():
            user_peer = update.get_effective_user()
            user_id = user_peer.peer_id

            if user_id in self.conversation_next_step_handlers:
                self.conversation_next_step_handlers.pop(user_id)

            if user_id in self.conversation_data:
                self.conversation_data.pop(user_id)

    def set_conversation_data(self, update, key, value):
        if isinstance(update, FatSeqUpdate) and update.is_message_update():
            user_peer = update.get_effective_user()
            user_id = user_peer.peer_id

            if user_id in self.conversation_data:
                new_dict_element = {key: value}
                if isinstance(self.conversation_data[user_id], dict):
                    self.conversation_data[user_id].update(new_dict_element)
            else:
                self.conversation_data[user_id] = {key: value}

    def unset_conversation_data(self, update, key):
        if isinstance(update, FatSeqUpdate) and update.is_message_update():
            user_peer = update.get_effective_user()
            user_id = user_peer.peer_id

            if user_id in self.conversation_data:
                if isinstance(self.conversation_data[user_id], dict):
                    if key in self.conversation_data[user_id]:
                        del self.conversation_data[user_id][key]

    def clear_conversation_data(self, update):
        if isinstance(update, FatSeqUpdate) and update.is_message_update():
            user_peer = update.get_effective_user()
            user_id = user_peer.peer_id

            if user_id in self.conversation_data:
                if isinstance(self.conversation_data[user_id], dict):
                    self.conversation_data[user_id].clear()

    def get_conversation_data(self, update, key):
        if isinstance(update, FatSeqUpdate) and update.is_message_update():
            user_peer = update.get_effective_user()
            user_id = user_peer.peer_id

            if user_id in self.conversation_data:
                if isinstance(self.conversation_data[user_id], dict):
                    return self.conversation_data[user_id].get(key, None)
            else:
                return None

    def get_difference_success_callback(self, response, user_data):
        if len(response.body.updates) > 0:
            for update in response.body.updates:
                self.process_fat_seq_update(update)
                self.last_seq = update.seq
                with open('last_seq.txt', 'w') as f:
                    f.write(str(self.last_seq))
        else:
            self.last_seq = response.body.seq
            with open('last_seq.txt', 'w') as f:
                f.write(str(self.last_seq))
        self.is_batch_updates_processed = True
        if response.body.need_more:
            self.get_updates()
        current_time = int(round(time.time() * 1000)) / 1000
        if current_time - self.last_poll_request_time >= self.timeInterval:
            self.poll()
        else:
            time.sleep(self.timeInterval - (current_time - self.last_poll_request_time))
            self.poll()

    def get_difference_failure_callback(self, response, user_data):
        self.is_batch_updates_processed = True
        current_time = int(round(time.time() * 1000)) / 1000
        if current_time - self.last_poll_request_time >= self.timeInterval:
            self.poll()
        else:
            time.sleep(self.timeInterval - (current_time - self.last_poll_request_time))
            self.poll()

    def get_updates(self):
        if self.is_batch_updates_processed:
            self.is_batch_updates_processed = False
            self.last_poll_request_time = int(round(time.time() * 1000)) / 1000
            self.bot.get_difference(self.last_seq, self.updates_number, self.get_difference_success_callback,
                                    self.get_difference_failure_callback)

    def get_last_seq_success_callback(self, response, user_data):
        self.last_seq = response.body.seq
        with open('last_seq.txt', 'w') as f:
            f.write(str(self.last_seq))
        self.get_updates()

    def get_last_seq(self):
        self.bot.get_last_seq(self.get_last_seq_success_callback)

    def poll(self):
        if not self.real_time_fetch_updates:
            if self.continue_last_processed_seq:
                if self.last_seq:
                    self.get_updates()
                else:
                    if Path("last_seq.txt").exists():
                        with open('last_seq.txt', 'r') as f:
                            self.last_seq = int(f.readline())
                            self.get_updates()
                    else:
                        self.get_last_seq()
            else:
                self.get_last_seq()



from balebot.handlers.handler import Handler
from balebot.handlers.command_handler import CommandHandler
from balebot.handlers.message_handler import MessageHandler
from balebot.handlers.quoted_message_handler import QuotedMessageHandler



import json as json_handler

from balebot.models.base_models.value_types.map_value_item import MapValueItem
from balebot.models.constants.errors import Error

from balebot.models.base_models.value_types.raw_value import RawValue


class MapValue(RawValue):
    def __init__(self, items):

        if isinstance(items, dict) and all(isinstance(value, RawValue) for key, value in items):

            items_list = []
            for key, value in items:
                temp_dict = {
                    "key": str(key),
                    "value": value,
                }
                items_list.append(MapValueItem.load_from_json(temp_dict))

            self.items = items_list

        elif all(isinstance(item, MapValueItem) for item in items):

            self.items = items
        else:
            raise ValueError(Error.unacceptable_object_type)

    def get_json_object(self):

        data = {
            "$type": "MapValue",
            "items": [item.get_json_object() for item in self.items],

        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        temp_items = json_dict.get('items', None)
        items = [MapValueItem.load_from_json(item) for item in temp_items]

        return cls(items=items)

    def find_value(self, key):
        for item in self.items:
            if item.key == key:
                return item.value
        return




__title__ = 'balebot'
__description__ = 'Python framework for Bale messenger Bot API'
__url__ = 'https://github.com/balemessenger/bale-bot-python'
__version__ = '1.6.12'
__author__ = 'bale'
__author_email__ = 'balebot@elenoon.ir'
__license__ = 'Apache 2.0'



from balebot.models.messages import contact_message
from balebot.models.messages import location_message
from balebot.models.constants.raw_json_type import RawJsonType



class RawJsonFactory:
    @staticmethod
    def create_raw_json(json_dict):
        json_type = json_dict.get("dataType", None)

        if json_type == RawJsonType.contact:
            return contact_message.ContactMessage.load_from_json(json_dict)

        elif json_type == RawJsonType.location:
            return location_message.LocationMessage.load_from_json(json_dict)

        else:
            return None



from enum import Enum


class MoneyRequestType(Enum):
    normal = "MoneyRequestNormal"
    bill = "MoneyRequestBill"
    payment = "MoneyRequestPayment"

    def __str__(self):
        return self.value




import json as json_handler

from balebot.models.base_models.value_types.raw_value import RawValue
from balebot.models.constants.errors import Error

from balebot.models.constants.value_type import ValueType


class Int64Val(RawValue):
    def __init__(self, value):
        self.value = str(value)

    def get_json_object(self):
        data = {
            "$type": ValueType.int64_val,
            "value": self.value,

        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        value = json_dict.get('value', None)

        return cls(value=value)



import asyncio
import pickle
import traceback
from collections import namedtuple

import redis

from balebot.config import Config
from balebot.dispatcher import Dispatcher
from balebot.utils.logger import Logger

redis_db = redis.StrictRedis(host=Config.redis_host, port=Config.redis_port, db=Config.redis_db) \
    if Config.state_holder else None
BotState = namedtuple('BotState', ['conversation_next_step_handlers', 'conversation_data'])


class Updater:
    def __init__(self, token, loop=None):

        self.logger = Logger.get_logger()

        if not token:
            raise ValueError("`token` did't passed")

        self.token = token
        self.timeout = Config.request_timeout

        self.bale_futures = []

        self._loop = asyncio.get_event_loop() if not loop else loop

        self.dispatcher = Dispatcher(loop=self._loop,
                                     token=self.token,
                                     bale_futures=self.bale_futures)

        self.running = False

    def run(self, stop_after=None):
        if Config.state_holder:
            bot_previous_state = redis_db.get(self.token)
            bot_previous_state = pickle.loads(bot_previous_state) if bot_previous_state else None
            self.dispatcher.conversation_next_step_handlers, self.dispatcher.conversation_data = \
                (bot_previous_state.conversation_next_step_handlers, bot_previous_state.conversation_data) \
                if bot_previous_state else ({}, {})

        asyncio.ensure_future(self._run_dispatcher())
        asyncio.ensure_future(self.dispatcher.bot.network.run())

        try:
            if isinstance(stop_after, int) or isinstance(stop_after, float):
                self._stop_after(stop_after)

            self._loop.run_forever()

        except Exception as e:
            self.logger.error("exception :  {} ,  {}".format(type(e), e), extra={"tag": "err"})
            traceback.print_exc()
        finally:
            self.stop()

    def stop(self):
        if Config.state_holder:
            redis_db.set(self.token, pickle.dumps(
                BotState(conversation_next_step_handlers=self.dispatcher.conversation_next_step_handlers,
                         conversation_data=self.dispatcher.conversation_data)))
        self.dispatcher.bot.network.stop_network()
        self._stop_dispatcher()
        self._loop.stop()

    def _run_dispatcher(self):
        return self.dispatcher.run()

    def _stop_dispatcher(self):
        self.dispatcher.stop()

    def _stop_after(self, delay):
        self._loop.call_later(delay=delay, callback=self.stop)

    def network_connected(self):
        return self.dispatcher.bot.network.connected()

    def connect_network(self):
        asyncio.ensure_future(self.dispatcher.bot.network.connect())



import json as json_handler

from balebot.models.base_models.jsonable import Jsonable
from balebot.models.constants.errors import Error

from balebot.models.base_models.file_location import FileLocation


class ImageLocation(Jsonable):
    def __init__(self, width, height, file_size, file_location):
        self.width = int(width)
        self.height = int(height)
        self.file_size = int(file_size)
        if isinstance(file_location, FileLocation):
            self.file_location = file_location
        else:
            raise ValueError(Error.unacceptable_object_type)

    def get_json_object(self):
        data = {
            "width": self.width,
            "height": self.height,
            "fileSize": self.file_size,
            "fileLocation": self.file_location.get_json_object()
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        width = json_dict.get("width", None)
        height = json_dict.get("height", None)
        file_size = json_dict.get("fileSize", None)
        file_location = FileLocation.load_from_json(json_dict.get("fileLocation", None))

        if (width is None) or (height is None) or (file_size is None) or (file_location is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(width=width, height=height, file_size=file_size, file_location=file_location)



import json as json_handler

from balebot.models.base_models.banking.bank_ext import BankExt
from balebot.models.constants.errors import Error
from balebot.models.constants.message_type import MessageType
from balebot.models.factories import bank_ext_factory
from balebot.models.messages.base_message import BaseMessage


class BankMessage(BaseMessage):
    def __init__(self, bank_ext_message):

        if isinstance(bank_ext_message, BankExt):
            self.bank_ext_message = bank_ext_message
        else:
            raise ValueError(Error.unacceptable_object_type)

    def get_receipt(self):
        from balebot.models.base_models.banking.receipt_message import ReceiptMessage
        if isinstance(self.bank_ext_message, ReceiptMessage):
            return self.bank_ext_message.generate_receipt_from_transfer_info_items()

    def get_json_object(self):

        data = {
            "$type": MessageType.bank_message,
            "message": self.bank_ext_message.get_json_object(),
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        bank_ext_message = bank_ext_factory.BankExtFactory.create_bank_ext(json_dict.get('message', None))

        if bank_ext_message is None:
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(bank_ext_message=bank_ext_message)



import json as json_handler

from balebot.models.base_models.request_body import RequestBody
from balebot.models.constants.request_type import RequestType


class GetGroupApiStruct(RequestBody):
    def __init__(self, group_id, client_user_id):
        self.group_id = str(group_id)
        self.client_user_id = str(client_user_id)

    def get_json_object(self):
        data = {
            "$type": RequestType.get_group_api_struct,
            "groupId": self.group_id,
            "clientUserId": self.client_user_id,

        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())



from balebot.models.messages.unsupported_message import UnsupportedMessage
from balebot.filters.filter import Filter


class UnsupportedFilter(Filter):
    def match(self, message):
        return isinstance(message, UnsupportedMessage)



from balebot.models.base_models.value_types.map_value import MapValue
from balebot.models.base_models.value_types.map_value_item import MapValueItem
from balebot.models.base_models.value_types.raw_value import RawValue
from balebot.models.base_models.value_types.string_val import StringVal
from balebot.models.base_models.value_types.array_val import ArrayVal
from balebot.models.base_models.value_types.boolean_value import BooleanValue
from balebot.models.base_models.value_types.double_val import DoubleVal
from balebot.models.base_models.value_types.int32_val import Int32Val
from balebot.models.base_models.value_types.int64_val import Int64Val



import json as json_handler

from balebot.models.constants.errors import Error


class BankAccount:
    def __init__(self, account_number, branch_code, open_date, rate, available_balance,
                 last_money_transfer_date, first_name, last_name):

        self._account_number = str(account_number)
        self._branch_code = int(branch_code)
        self._open_date = str(open_date)
        self._rate = int(rate)
        self._available_balance = str(available_balance)
        self._last_money_transfer_date = str(last_money_transfer_date)
        self._first_name = str(first_name)
        self._last_name = str(last_name)

    def get_json_object(self):
        data = {
            "accountNumber": self._account_number,
            "branchCode": self._branch_code,
            "openDate": self._open_date,
            "rate": self._rate,
            "availableBalance": self._available_balance,
            "lastMoneyTransferDate": self._last_money_transfer_date,
            "firstName": self._first_name,
            "lastName": self._last_name,

        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        account_number = json_dict.get('accountNumber', None)
        branch_code = json_dict.get('branchCode', None)
        open_date = json_dict.get('openDate', None)
        rate = json_dict.get('rate', None)
        available_balance = json_dict.get('availableBalance', None)
        last_money_transfer_date = json_dict.get('lastMoneyTransferDate', None)
        first_name = json_dict.get('firstName', None)
        last_name = json_dict.get('lastName', None)

        if (not account_number) or (branch_code is None) or (not open_date) or (rate is None) \
                or (available_balance is None) or (not last_money_transfer_date) \
                or (not first_name) or (not last_name):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(account_number=account_number, branch_code=branch_code, open_date=open_date,
                   rate=rate, available_balance=available_balance,
                   last_money_transfer_date=last_money_transfer_date,
                   first_name=first_name, last_name=last_name)



class ServiceType:
    Messaging = 'messaging'
    Files = 'files'
    Stickers = 'stickers'
    Users = 'users'
    KeyValue = 'keyvalue'
    Bots = 'bots'
    WebHooks = 'webhooks'
    Monitor = 'monitor'
    Groups = "groups"
    SequenceUpdate = "sequence-update"
    banks = "banks"



from balebot.models.messages.video_message import VideoMessage
from balebot.filters.filter import Filter


class VideoFilter(Filter):
    def match(self, message):
        if isinstance(message, VideoMessage):
            return self.validator(message) if self.validator else True
        else:
            return False



import json as json_handler

from balebot.models.base_models.peer import Peer
from balebot.models.constants.errors import Error

from balebot.models.server_updates.update_body import UpdateBody


class BotReadUpdate(UpdateBody):
    def __init__(self, json_dict):
        if isinstance(json_dict, dict):
            json_dict = json_dict
        elif isinstance(json_dict, str):
            json_dict = json_handler.loads(json_dict)
        else:
            raise ValueError(Error.unacceptable_json)

        self.peer = Peer.load_from_json(json_dict.get("peer", None))
        self.start_date = json_dict.get("startDate", None)
        self.read_date = json_dict.get("readDate", None)



import json as json_handler

from balebot.models.constants.document_type import DocumentType
from balebot.models.constants.message_type import MessageType
from balebot.models.messages.base_message import BaseMessage
from balebot.models.constants.errors import Error

from balebot.models.messages.text_message import TextMessage


class DocumentMessage(BaseMessage):
    def __init__(self, file_id, access_hash, name, file_size, mime_type, caption_text=None,
                 checksum=None, algorithm=None, file_storage_version=1):
        self.file_id = str(file_id)
        self.access_hash = str(access_hash)
        self.file_size = str(file_size)
        self.name = str(name)
        self.mime_type = str(mime_type)

        if caption_text:
            if isinstance(caption_text, TextMessage):
                self.caption_text = caption_text
        else:
            self.caption_text = None

        self.checksum = str(checksum) if checksum else "checkSum"
        self.algorithm = str(algorithm) if algorithm else "algorithm"
        self.file_storage_version = int(file_storage_version)

    def get_json_object(self):
        data = {
            "$type": MessageType.document_message,
            "fileId": self.file_id,
            "accessHash": self.access_hash,
            "fileSize": self.file_size,
            "name": self.name,
            "mimeType": self.mime_type,
            "thumb": None,
            "ext": None,
            "caption": self.caption_text.get_json_object() if self.caption_text else None,
            "checkSum": self.checksum,
            "algorithm": self.algorithm,
            "fileStorageVersion": self.file_storage_version

        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        file_id = json_dict.get('fileId', None)
        access_hash = json_dict.get('accessHash', None)
        file_size = json_dict.get('fileSize', None)
        name = json_dict.get('name', None)
        mime_type = json_dict.get('mimeType', None)

        caption_message = TextMessage.load_from_json(json_dict.get('caption', None))

        checksum = json_dict.get('checkSum', None)
        algorithm = json_dict.get('algorithm', None)
        file_storage_version = json_dict.get('fileStorageVersion', None)

        if (not file_id) or (not access_hash) or (name is None) or (file_size is None) or (mime_type is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(file_id=file_id, access_hash=access_hash, name=name, file_size=file_size,
                   mime_type=mime_type, file_storage_version=file_storage_version, caption_text=caption_message,
                   checksum=checksum, algorithm=algorithm)

    @staticmethod
    def is_video_message(json_dict):
        message_type = json_dict.get("$type", None)
        message_ext = json_dict.get("ext", None)
        document_type = message_ext.get("$type", None)

        return message_type == MessageType.document_message and document_type == DocumentType.video_document

    @staticmethod
    def is_photo_message(json_dict):
        message_type = json_dict.get("$type", None)
        message_ext = json_dict.get("ext", None)
        document_type = message_ext.get("$type", None)

        return message_type == MessageType.document_message and document_type == DocumentType.photo_document

    @staticmethod
    def is_voice_message(json_dict):
        message_type = json_dict.get("$type", None)
        message_ext = json_dict.get("ext", None)
        document_type = message_ext.get("$type", None)

        return message_type == MessageType.document_message and document_type == DocumentType.voice_document

    @staticmethod
    def is_raw_document_message(json_dict):
        message_type = json_dict.get("$type", None)
        message_ext = json_dict.get("ext", None)

        return message_type == MessageType.document_message and not message_ext



import json as json_handler

from balebot.models.base_models.response_body import ResponseBody
from balebot.models.constants.errors import Error


class MessageEdited(ResponseBody):
    def __init__(self, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        self.date = json_dict.get("date", None)
        self.seq = json_dict.get("seq", None)



from balebot.models.base_models.request_body import RequestBody


class RequestToResponseMapper:
    request_response_map = {
        "GetFileDownloadUrl": (
            "balebot.models.server_responses.files.response_get_download_url", "ResponseGetDownloadUrl"),
        "GetFileUploadUrl": ("balebot.models.server_responses.files.response_get_upload_url", "ResponseGetUploadUrl"),

        "CreateGroup": ("balebot.models.server_responses.group.response_create_group", "ResponseCreateGroup"),
        "GetGroupApiStruct": ("balebot.models.server_responses.group.get_group_api_response", "GetGroupApiResponse"),
        "InviteUser": ("balebot.models.server_responses.bot_success_response", "BotSuccess"),

        "SendMessage": ("balebot.models.server_responses.messaging.message_sent", "MessageSent"),
        "EditMessage": (
            "balebot.models.server_responses.messaging.message_edited", "MessageEdited"),

        "GetDifference": ("balebot.models.server_responses.sequence.difference_update", "DifferenceUpdate"),
        "GetLastSequence": ("balebot.models.server_responses.sequence.get_last_seq_response", "GetLastSeqResponse"),

    }

    @classmethod
    def get_response(cls, request):
        if isinstance(request, RequestBody):
            request_class_name = request.__class__.__name__
            return cls.request_response_map[str(request_class_name)]
        else:
            return cls.request_response_map[str(request)]



import json as json_handler

from balebot.models.base_models.banking.bank_ext import BankExt
from balebot.models.base_models.value_types.map_value import MapValue
from balebot.models.constants.bank_ext_types import BankExtTypes
from balebot.models.constants.errors import Error
from collections import namedtuple

from balebot.models.factories.raw_value_factory import RawValueFactory


class ReceiptMessage(BankExt):
    def __init__(self, message, transfer_info):

        if isinstance(message, MapValue):
            self.message = message
        else:
            raise ValueError(Error.unacceptable_object_type)

        if isinstance(transfer_info, MapValue):
            self.transfer_info = transfer_info
        else:
            raise ValueError(Error.unacceptable_object_type)

    def get_json_object(self):
        data = {
            "$type": BankExtTypes.deprecated_receipt,
            "message": self.message.get_json_object(),
            "transferInfo": self.transfer_info.get_json_object(),
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    def generate_receipt_from_transfer_info_items(self):
        BankReceipt = namedtuple('BankReceipt', [item.key for item in self.transfer_info.items])
        params = [RawValueFactory.get_value(self.transfer_info.items[i].value)
                  for i in range(self.transfer_info.items.__len__())]
        bank_receipt = BankReceipt(*params)
        return bank_receipt

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        message = MapValue.load_from_json(json_dict.get('message', None))
        transfer_info = MapValue.load_from_json(json_dict.get('transferInfo', None))

        if (message is None) or (transfer_info is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(message=message, transfer_info=transfer_info)



from balebot.filters.default import DefaultFilter
from balebot.filters.document_filter import DocumentFilter
from balebot.filters.filter import Filter
from balebot.filters.json_filter import JsonFilter
from balebot.filters.photo_filter import PhotoFilter
from balebot.filters.sticker_filter import StickerFilter
from balebot.filters.text_filter import TextFilter
from balebot.filters.unsupported_filter import UnsupportedFilter
from balebot.filters.video_filter import VideoFilter
from balebot.filters.voice_filter import VoiceFilter
from balebot.filters.template_response_filter import TemplateResponseFilter
from balebot.filters.contact_filter import ContactFilter
from balebot.filters.location_filter import LocationFilter
from balebot.filters.bank_message_filter import BankMessageFilter
from balebot.filters.template_message_filter import TemplateMessageFilter




import json as json_handler

from balebot.models.base_models.jsonable import Jsonable
from balebot.models.constants.errors import Error


class PackInfo(Jsonable):
    def __init__(self, pack_id, name):
        self.pack_id = str(pack_id)
        self.name = str(name)

    def get_json_object(self):
        data = {
            "id": self.pack_id,
            "name": self.name,
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        pack_id = json_dict.get('id', None)
        name = json_dict.get('name', None)

        if (not pack_id) or (name is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(pack_id=pack_id, name=name)



from balebot.models.messages.voice_message import VoiceMessage
from balebot.filters.filter import Filter


class VoiceFilter(Filter):
    def match(self, message):
        if isinstance(message, VoiceMessage):
            return self.validator(message) if self.validator else True
        else:
            return False



import json as json_handler
from balebot.models.constants.errors import Error

from balebot.models.base_models.avatar import Avatar
from balebot.models.base_models.group_member import GroupMember

from balebot.models.base_models.jsonable import Jsonable


class Group(Jsonable):
    def __init__(self, group_id, access_hash, title, is_member, creator_user_id, members,
                 about=None, avatar=None):
        self.id = str(group_id)
        self.access_hash = str(access_hash)
        self.title = str(title)
        self.is_member = bool(is_member)
        self.creator_user_id = str(creator_user_id)

        if all(isinstance(member, GroupMember) for member in members):
            self.members = members
        else:
            raise ValueError(Error.unacceptable_object_type)

        self.about = str(about) if about else None

        if avatar:
            if isinstance(avatar, Avatar):
                self.avatar = avatar
            else:
                raise ValueError(Error.unacceptable_object_type)
        else:
            self.avatar = None

    def get_json_object(self):
        data = {
            "id": self.id,
            "accessHash": self.access_hash,
            "title": self.title,
            "isMember": self.is_member,
            "creatorUserId": self.creator_user_id,
            "members": self.members,
            "about": self.about,
            "avatar": self.avatar,
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        group_id = json_dict.get('id', None)
        access_hash = json_dict.get('accessHash', None)
        title = json_dict.get('title', None)
        is_member = json_dict.get('isMember', None)
        creator_user_id = json_dict.get('creatorUserId', None)
        members = [GroupMember.load_from_json(member) for member in json_dict.get('members', None)]

        if (not group_id) or (not access_hash) or (title is None) or (is_member is None) or (
                not creator_user_id) or (members is None):
            raise ValueError(Error.none_or_invalid_attribute)

        about = json_dict.get('about', None)
        avatar = Avatar.load_from_json(json_dict.get('avatar', None)) if json_dict.get('avatar', None) else None

        return cls(group_id=group_id, access_hash=access_hash, is_member=is_member, creator_user_id=creator_user_id,
                   members=members, title=title, about=about, avatar=avatar)



import os

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

requires = [
        'aiohttp==2.3.7',
        'asyncio==3.4.3',
        'graypy==0.2.14',
        'backoff==1.6.0',
        'pillow==5.2.0',
        'redis==3.0.1'
        ]

about = {}
with open(os.path.join(here, 'balebot', '__version__.py'), mode='rt', encoding='utf-8') as f:
    exec(f.read(), about)

setup(
        name=about['__title__'],
        version=about['__version__'],
        description=about['__description__'],
        author=about['__author__'],
        author_email=about['__author_email__'],
        license=about['__license__'],
        long_description="\n\n".join([open("README.md").read(), open("CHANGELOG.md").read()]),
        long_description_content_type='text/markdown',
        url=about['__url__'],
        install_requires=requires,
        packages=find_packages(),
)



import unittest
from unittest.mock import MagicMock

from balebot.models.base_models import Peer
from balebot.models.client_requests import EditMessage
from balebot.models.messages import TextMessage


class TestEditMessage(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        peer = Peer(peer_type="User", peer_id=1422775695,
                    access_hash="asdqweqwdasd")
        message = TextMessage("ASD")
        cls.update_request = EditMessage(updated_message=message, peer=peer, random_id=123465)

    def test_get_json_object(self):
        self.assertIsInstance(self.update_request.get_json_object(), dict)

    def test_get_json_str(self):
        self.assertIsInstance(self.update_request.get_json_str(), str)



import json as json_handler

from balebot.models.messages.document_message import DocumentMessage
from balebot.models.constants.errors import Error

from balebot.models.constants.document_type import DocumentType
from balebot.models.messages.text_message import TextMessage


class VoiceMessage(DocumentMessage):
    def __init__(self, file_id, access_hash, name, file_size, mime_type, duration, file_storage_version=None,
                 caption_text=None, checksum=None, algorithm=None):

        super(VoiceMessage, self).__init__(file_id, access_hash, name, file_size, mime_type,
                                           caption_text, checksum, algorithm, file_storage_version)
        self.duration = int(duration)

    def get_json_object(self):

        data = super().get_json_object()
        data["ext"] = {
            "$type": DocumentType.voice_document,
            "duration": self.duration
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        file_id = json_dict.get('fileId', None)
        access_hash = json_dict.get('accessHash', None)
        name = json_dict.get('name', None)
        file_size = json_dict.get('fileSize', None)
        mime_type = json_dict.get('mimeType', None)

        ext = json_dict.get('ext', None)
        duration = ext.get('duration', None)

        file_storage_version = json_dict.get('fileStorageVersion', None)

        caption_message = TextMessage.load_from_json(json_dict.get('caption', None))

        checksum = json_dict.get('checkSum', None)
        algorithm = json_dict.get('algorithm', None)

        if (not file_id) or (not access_hash) or (not name) or (file_size is None) or (not mime_type) or (
                    duration is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(file_id=file_id, access_hash=access_hash, name=name, file_size=file_size,
                   mime_type=mime_type, duration=duration, file_storage_version=file_storage_version,
                   caption_text=caption_message, checksum=checksum, algorithm=algorithm)



import json as json_handler

from balebot.models.server_updates.update_body import UpdateBody
from balebot.models.constants.errors import Error


class RawUpdate(UpdateBody):
    def __init__(self, json_dict):
        if isinstance(json_dict, dict):
            json_dict = json_dict
        elif isinstance(json_dict, str):
            json_dict = json_handler.loads(json_dict)
        else:
            raise ValueError(Error.unacceptable_json)

        self.data = json_dict.get("data", None)



import json as json_handler

from balebot.models.constants.peer_type import PeerType
from .peer import Peer


class GroupPeer(Peer):
    def __init__(self, peer_id, access_hash):
        super(GroupPeer, self).__init__(peer_type=PeerType.group, peer_id=peer_id, access_hash=access_hash)

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError("json input has unacceptable format.")

        peer_type = json_dict.get('$type', None)
        peer_id = json_dict.get('id', None)
        access_hash = json_dict.get('accessHash', None)

        if (peer_type != PeerType.group) or (not peer_id) or (not access_hash):
            raise ValueError("peer_type , peer_id and access_hash should have correct values.")

        return cls(peer_id=peer_id, access_hash=access_hash)



import json as json_handler

from balebot.models.base_models.raw_json import RawJson
from balebot.models.constants.errors import Error
from balebot.models.constants.raw_json_type import RawJsonType


class Contact(RawJson):
    def __init__(self, name, emails, phones):
        self.name = str(name)

        if isinstance(emails, list):
            self.emails = [str(email) for email in emails]
        else:
            raise ValueError(Error.unacceptable_object_type)

        if isinstance(phones, list):
            self.phones = ["".join(str(phone).split(" ")) for phone in phones]
        else:
            raise ValueError(Error.unacceptable_object_type)

    def get_json_object(self):
        data = {
            "dataType": RawJsonType.contact,
            "data": {
                RawJsonType.contact: {
                    "name": self.name,
                    "emails": self.emails,
                    "phones": self.phones
                }
            }
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        data = json_dict.get('data', None)
        contact = data.get(RawJsonType.contact, None)
        name = contact.get('name', None)
        emails = contact.get('emails', None)
        phones = contact.get('phones', None)

        return cls(name=name, emails=emails, phones=phones)



from balebot.models.messages.banking.bank_message import BankMessage
from balebot.filters.filter import Filter


class BankMessageFilter(Filter):
    def match(self, message):
        if isinstance(message, BankMessage):
            return self.validator(message) if self.validator else True
        else:
            return False



import json as json_handler

from balebot.models.constants.raw_json_type import RawJsonType
from balebot.models.base_models.raw_json import RawJson
from balebot.models.messages.base_message import BaseMessage
from balebot.models.constants.errors import Error
from balebot.models.constants.message_type import MessageType


class LocationMessage(RawJson, BaseMessage):
    def __init__(self, latitude, longitude):
        self.latitude = float(latitude)
        self.longitude = float(longitude)

    def get_json_object(self):

        data = {
            "$type": MessageType.json_message,
            "rawJson": json_handler.dumps({
                "dataType": RawJsonType.location,
                "data": {
                    RawJsonType.location: {
                        "latitude": self.latitude,
                        "longitude": self.longitude
                    }
                }
            })
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        json_dict = json_handler.loads(json_dict.get('rawJson'))
        data = json_dict.get('data', None)
        location = data.get(RawJsonType.location, None)
        latitude = location.get('latitude', None)
        longitude = location.get('longitude', None)

        return cls(latitude=latitude, longitude=longitude)



import json as json_handler

from balebot.models.constants.peer_type import PeerType
from balebot.models.constants.errors import Error

from .peer import Peer


class UserPeer(Peer):
    def __init__(self, peer_id, access_hash):
        super(UserPeer, self).__init__(peer_type=PeerType.user, peer_id=peer_id, access_hash=access_hash)

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        peer_type = json_dict.get('$type', None)
        peer_id = json_dict.get('id', None)
        access_hash = json_dict.get('accessHash', None)

        if (peer_type != PeerType.user) or (not peer_id) or (not access_hash):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(peer_id=peer_id, access_hash=access_hash, )



import asyncio
import traceback
import aiohttp
import backoff

from balebot.config import Config
from balebot.utils.logger import Logger


class Network:
    """ network layer main class """

    def __init__(self, token, incoming_queue=None, outgoing_queue=None, loop=None):

        self.logger = Logger.get_logger()
        self._base_url = Config.base_url
        self._token = token
        self._running = False
        self._incoming_queue = incoming_queue or asyncio.Queue()
        self._outgoing_queue = outgoing_queue or asyncio.Queue()
        self._session = None
        self._ws = None
        self._loop = loop

        self._listener_task = None
        self._sender_task = None
        self._heartbeat = Config.heartbeat
        self._receive_timeout = Config.receive_timeout

    @backoff.on_predicate(backoff.fibo)
    async def connect(self):
        if self._ws is None:

            try:
                self._session = aiohttp.ClientSession(loop=self._loop)
                self._ws = await self._session.ws_connect(self.construct_url(),heartbeat=self._heartbeat,receive_timeout=self._receive_timeout,headers={"source":"python3.5"})
                self.logger.warning('connect success: {}'.format(self.construct_url()))
            except Exception as e:
                await self.disconnect()
                self.logger.error('connect error: {}'.format(e),
                                  extra={"tag": "err","type": "connection", "url": self.construct_url(), "error_type": type(e)})
                traceback.print_exc()

        elif self._ws.closed:
            try:
                if self._session.closed:
                    self._session = aiohttp.ClientSession(loop=self._loop)
                self._ws = await self._session.ws_connect(self.construct_url(),heartbeat=self._heartbeat,receive_timeout=self._receive_timeout,headers={"source":"python3.5"})
                self.logger.warning('reconnect success: {}'.format(self.construct_url()))
            except Exception as e:
                await self.disconnect()
                self.logger.error('reconnect error: {}'.format(e),
                                  extra={"tag": "err", "type": "connection", "url": self.construct_url(), "error_type": type(e)})
                traceback.print_exc()

        return self._ws

    def construct_url(self):
        return str(self._base_url + self._token)

    async def process(self):

        try:

            if await self.connect():

                self._listener_task = asyncio.ensure_future(self._ws.receive())
                self._sender_task = asyncio.ensure_future(self._outgoing_queue.get())

                main_tasks = [self._listener_task, self._sender_task]

                done, pending = await asyncio.wait(main_tasks, return_when=asyncio.FIRST_COMPLETED)

                if self._listener_task in done:
                    self.logger.debug('[transport] receiving :  {}'.format(self._listener_task.result()))
                    await self._incoming_queue.put(self._listener_task.result())
                else:
                    # self.logger.debug("listener task pended")
                    self._listener_task.cancel()

                if self._sender_task in done:
                    message = self._sender_task.result()
                    self._ws.send_str(str(message))
                    self.logger.debug('[transport] sending :  {}'.format(message))
                elif self._sender_task:
                    # self.logger.debug("sender tast pended")
                    self._sender_task.cancel()

        except Exception as ex:
            self.logger.error(ex, extra={"tag": "err"})
            traceback.print_exc()

    async def run(self):

        self._running = True
        while self._running:
            await self.process()
            await asyncio.sleep(0)

        if self._listener_task:
            self._listener_task.cancel()

        if self._sender_task:
            self._sender_task.cancel()

        await self.disconnect()

    def stop_network(self):
        self.logger.warning("network stopped working.")
        self._running = False

    async def disconnect(self):
        if self._ws:
            await self._ws.close()
        if self._session:
            await self._session.close()
        self.logger.warning("network connection disconnected.")

    def connected(self):
        if not self._session:
            return False
        elif not self._ws:
            return False
        elif self._session.closed or self._ws.closed:
            return False
        else:
            return True

    def send(self, item):
        self._outgoing_queue.put_nowait(item)


from balebot.filters.filter import Filter
from balebot.models.messages.document_message import DocumentMessage


class DocumentFilter(Filter):
    def match(self, message):
        return isinstance(message, DocumentMessage) and DocumentMessage.is_raw_document_message(
            message.get_json_object())



import unittest
from unittest.mock import MagicMock

from balebot.models.base_models import Peer
from balebot.models.messages import TextMessage
from balebot.bot import Bot
from balebot.models.base_models import Request


class TestBot(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.bot = MagicMock()
        cls.bot.set_future.return_value = None
        cls.bot.send_request.return_value = None

    def test_update_message_content(self):
        message = TextMessage("ASD")
        peer = Peer(peer_type="User", peer_id=1422775695,
                    access_hash="asdqweqwdasd")
        self.assertIsInstance(Bot.edit_message(self.bot, message=message, user_peer=peer, random_id=13246),
                              Request)



import json as json_handler

from balebot.models.base_models.request_body import RequestBody
from balebot.models.constants.errors import Error


class ResponseGetUploadUrl(RequestBody):
    def __init__(self, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        self.file_id = json_dict.get("fileId", None)
        self.user_id = json_dict.get("userId", None)
        self.url = json_dict.get("url", None)
        self.dup = json_dict.get("dup", None)



import json as json_handler

from balebot.models.constants.default_purchase_message_photo import DefaultPhoto
from balebot.models.constants.errors import Error
from balebot.models.constants.message_type import MessageType
from balebot.models.factories import message_factory
from balebot.models.messages.base_message import BaseMessage


class PurchaseMessage(BaseMessage):
    def __init__(self, account_number, amount, money_request_type, msg=None):
        from balebot.models.messages import PhotoMessage, TextMessage
        if msg and isinstance(msg, PhotoMessage):
            self.msg = msg
        elif msg and isinstance(msg, TextMessage):
            self.msg = PhotoMessage(file_id=DefaultPhoto.file_id, access_hash=DefaultPhoto.access_hash,
                                    name=DefaultPhoto.name, file_size=DefaultPhoto.file_size,
                                    mime_type=DefaultPhoto.mime_type, thumb=DefaultPhoto.thumb,
                                    caption_text=msg, width=DefaultPhoto.width, height=DefaultPhoto.height,
                                    ext_width=DefaultPhoto.ext_width, ext_height=DefaultPhoto.ext_height,
                                    file_storage_version=1)
        else:
            self.msg = PhotoMessage(file_id=DefaultPhoto.file_id, access_hash=DefaultPhoto.access_hash,
                                    name=DefaultPhoto.name, file_size=DefaultPhoto.file_size,
                                    mime_type=DefaultPhoto.mime_type, thumb=DefaultPhoto.thumb,
                                    caption_text=TextMessage(""), width=DefaultPhoto.width, height=DefaultPhoto.height,
                                    ext_width=DefaultPhoto.ext_width, ext_height=DefaultPhoto.ext_height,
                                    file_storage_version=1)

        self.account_number = str(account_number)

        if amount:
            self.amount = str(amount)
            self.regex_amount = "[" + str(amount) + "]"
        else:
            self.amount = "0"
            self.regex_amount = "[]"

        self.money_request_type = str(money_request_type)

    def get_json_object(self):
        data = {
            "$type": MessageType.purchase_message,
            "msg": self.msg.get_json_object(),
            "accountNumber": self.account_number,
            "amount": self.amount,
            "regexAmount": self.regex_amount,
            "moneyRequestType": {
                "$type": self.money_request_type
            }
        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        msg = message_factory.MessageFactory.create_message(json_dict.get('msg', None))
        account_number = json_dict.get('accountNumber', None)
        amount = json_dict.get('amount', None)

        if (msg is None) or (not account_number) or (amount is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(msg=msg, account_number=account_number, amount=amount)


class PurchaseMessage_IPG(BaseMessage):
    def __init__(self, msg, amount, money_request_type, terminal_ids, cardacq_id, iban_number=None):

        if isinstance(msg, BaseMessage):
            self.msg = msg
        else:
            raise ValueError(Error.unacceptable_object_type)

        self.account_number = ''

        if amount:
            self.amount = str(amount)
            self.regex_amount = "[" + str(amount) + "]"
        else:
            self.amount = "0"
            self.regex_amount = "[]"

        self.money_request_type = str(money_request_type)
        self.terminalId = str(terminal_ids)
        self.cardAcqId = str(cardacq_id)
        if iban_number:
            self.ibanNumber = int(iban_number)
        else:
            self.ibanNumber = iban_number

    def get_json_object(self):
        data = {"$type": "PurchaseMessage",
                "msg": self.msg.get_json_object(),
                "accountNumber": self.account_number,
                "amount": self.amount,
                "regexAmount": self.regex_amount,
                "moneyRequestType": {
                    "$type": "MoneyRequestPayment"
                },

                "additionalInfo": {
                    "$type": "MapValue",
                    "items": [
                        {
                            "key": "terminalId",
                            "value": {
                                "$type": "StringVal",
                                "text": self.terminalId
                            }
                        },
                        {
                            "key": "cardAcqId",
                            "value": {
                                "$type": "StringVal",
                                "text": self.cardAcqId
                            }
                        }
                    ]
                }
                }
        if self.ibanNumber:
            data.update({"multiplexingData": {
                    "multiplexingType": {
                        "$type": "Amount"
                    },
                    "multiplexingRows": [
                        {
                            "ibanNumber": self.ibanNumber,
                            "value": int(self.amount)
                        }
                    ]
                }})
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        msg = message_factory.MessageFactory.create_message(json_dict.get('msg', None))
        account_number = json_dict.get('accountNumber', None)
        amount = json_dict.get('amount', None)

        if (msg is None) or (not account_number) or (amount is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(msg=msg, account_number=account_number, amount=amount)



import json as json_handler

from balebot.models.messages.document_message import DocumentMessage
from balebot.models.constants.errors import Error

from balebot.models.constants.document_type import DocumentType
from balebot.models.messages.text_message import TextMessage


class VideoMessage(DocumentMessage):
    def __init__(self, file_id, access_hash, name, file_size, mime_type, thumb, duration, width=80, height=80,
                 ext_width=None, ext_height=None, file_storage_version=None, caption_text=None, checksum=None,
                 algorithm=None):

        super(VideoMessage, self).__init__(file_id, access_hash, name, file_size, mime_type,
                                           caption_text, checksum, algorithm, file_storage_version)

        self.thumb = str(thumb)
        self.duration = int(duration)
        self.width = int(width)
        self.height = int(height)
        self.ext_width = int(ext_width) if ext_width else width
        self.ext_height = int(ext_height) if ext_height else height

    def get_json_object(self):

        data = super().get_json_object()
        data["thumb"] = {
            "width": self.width,
            "height": self.height,
            "thumb": self.thumb
        }
        data["ext"] = {
            "$type": DocumentType.video_document,
            "width": self.ext_width,
            "height": self.ext_height,
            "duration": self.duration
        }

        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        file_id = json_dict.get('fileId', None)
        access_hash = json_dict.get('accessHash', None)
        name = json_dict.get('name', None)
        file_size = json_dict.get('fileSize', None)
        mime_type = json_dict.get('mimeType', None)

        ext = json_dict.get('ext', None)
        ext_width = ext.get('width', None)
        ext_height = ext.get('height', None)
        duration = ext.get('duration', None)

        thumb = json_dict.get('thumb', None)
        width = thumb.get('width', None)
        height = thumb.get('height', None)
        thumb_txt = thumb.get('thumb', None)

        file_storage_version = json_dict.get('fileStorageVersion', None)

        caption_message = TextMessage.load_from_json(json_dict.get('caption', None))

        checksum = json_dict.get('checkSum', None)
        algorithm = json_dict.get('algorithm', None)

        if (not file_id) or (not access_hash) or (name is None) or (file_size is None) \
                or (mime_type is None) or (width is None) or (height is None) or (ext_width is None) \
                or (ext_height is None) or (thumb_txt is None) or (duration is None):
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(file_id=file_id, access_hash=access_hash, name=name, file_size=file_size,
                   mime_type=mime_type, thumb=thumb_txt, duration=duration, width=width, height=height,
                   file_storage_version=file_storage_version, caption_text=caption_message, checksum=checksum,
                   algorithm=algorithm)



import json as json_handler

from balebot.models.constants.message_type import MessageType
from balebot.models.messages.base_message import BaseMessage
from balebot.models.constants.errors import Error

from balebot.models.messages.template.template_message_button import TemplateMessageButton


class TemplateMessage(BaseMessage):
    message_id_counter = 0

    def __init__(self, general_message, btn_list):

        if isinstance(general_message, BaseMessage):
            self.general_message = general_message
        else:
            raise ValueError(Error.unacceptable_object_type)

        if isinstance(btn_list, list) and all(isinstance(btn, TemplateMessageButton) for btn in btn_list):
            self.btn_list = btn_list
        elif btn_list is None:
            self.btn_list = []
        else:
            raise ValueError(Error.unacceptable_object_type)

        self.template_message_id = str(TemplateMessage.message_id_counter)
        TemplateMessage.message_id_counter += 1

    def get_json_object(self):
        data = {
            "$type": MessageType.template_message,
            "generalMessage": self.general_message.get_json_object(),
            "templateMessageId": self.template_message_id,
            "btnList": [btn.get_json_object() for btn in self.btn_list],

        }
        return data

    def get_json_str(self):
        return json_handler.dumps(self.get_json_object())

    def add_button(self, btn):
        if isinstance(btn, TemplateMessageButton):
            self.btn_list.append(btn)
        else:
            raise ValueError(Error.unacceptable_object_type)

    @classmethod
    def load_from_json(cls, json):
        if isinstance(json, dict):
            json_dict = json
        elif isinstance(json, str):
            json_dict = json_handler.loads(json)
        else:
            raise ValueError(Error.unacceptable_json)

        from balebot.models.factories import message_factory
        general_message = message_factory.MessageFactory.create_message(json_dict.get("generalMessage"))
        bt_list = json_dict.get('btnList')
        btn_list = [TemplateMessageButton(btn['text'], btn['value'], 0) for btn in bt_list]
        if general_message is None or bt_list is None:
            raise ValueError(Error.none_or_invalid_attribute)

        return cls(general_message=general_message, btn_list=btn_list)
